@@ -498,10 +498,10 @@ cpl_error_code kmclipm_rtd_image_from_memory(const cpl_image *actual_img,
             (strcmp(getenv("KMCLIPM_DEBUG"), "") != 0))
         {
             /* $KMCLIPM_DEBUG is set, so write file */
-            system("mkdir -p $KMCLIPM_DEBUG/tmp");
-            system("echo '-----------------------------------------------------"
-                   "----' >> $KMCLIPM_DEBUG/tmp/kmclipm_fitpar.txt");
-            system("date >> $KMCLIPM_DEBUG/tmp/kmclipm_fitpar.txt");
+            if (system("mkdir -p $KMCLIPM_DEBUG/tmp"));
+            if (system("echo '-----------------------------------------------------"
+                   "----' >> $KMCLIPM_DEBUG/tmp/kmclipm_fitpar.txt"));
+            if (system("date >> $KMCLIPM_DEBUG/tmp/kmclipm_fitpar.txt"));
             char *cmd = cpl_sprintf("echo 'Filt: %s, grat: %s, angle: %g, "
                                     "act_img: %p (type: %s), add_img: %p (type: %s), "
                                     "ifu_id: %p, nominal_pos: %p'"
@@ -510,9 +510,9 @@ cpl_error_code kmclipm_rtd_image_from_memory(const cpl_image *actual_img,
                                     actual_img, actual_img_type,
                                     additional_img, additional_img_type,
                                     ifu_id, nominal_pos);
-            system(cmd);
+            if (system(cmd));
             cpl_free(cmd); cmd = NULL;
-            system("echo '#IFU           "
+            if (system("echo '#IFU           "
                    "intens         "
                    "(err)            "
                    "x0             "
@@ -522,7 +522,7 @@ cpl_error_code kmclipm_rtd_image_from_memory(const cpl_image *actual_img,
                    "fwhm           "
                    "(err)            "
                    "offset         "
-                   "(err)' >> $KMCLIPM_DEBUG/tmp/kmclipm_fitpar.txt");
+                   "(err)' >> $KMCLIPM_DEBUG/tmp/kmclipm_fitpar.txt"));
         }
 
 /*------------------------------------------------------------------------------
@@ -1015,32 +1015,32 @@ cpl_error_code kmclipm_rtd_image_from_memory(const cpl_image *actual_img,
                                                         fitted_pars[8],
                                                         fitted_pars[4],
                                                         fitted_pars[9]);
-                                system(cmd);
+                                if (system(cmd));
                                 cpl_free(cmd);
                                 cmd = cpl_sprintf("echo '%12.6g' >>"
                                                   "$KMCLIPM_DEBUG/tmp/kmclipm_intensity.txt",
                                                   fitted_pars[0]);
-                                system(cmd);
+                                if (system(cmd));
                                 cpl_free(cmd);
                                 cmd = cpl_sprintf("echo '%12.6g' >>"
                                                   "$KMCLIPM_DEBUG/tmp/kmclipm_xcenter.txt",
                                                   fitted_pars[1]);
-                                system(cmd);
+                                if (system(cmd));
                                 cpl_free(cmd);
                                 cmd = cpl_sprintf("echo '%12.6g' >>"
                                                   "$KMCLIPM_DEBUG/tmp/kmclipm_ycenter.txt",
                                                   fitted_pars[2]);
-                                system(cmd);
+                                if (system(cmd));
                                 cpl_free(cmd);
                                 cmd = cpl_sprintf("echo '%12.6g' >>"
                                                   "$KMCLIPM_DEBUG/tmp/kmclipm_fwhm.txt",
                                                   fitted_pars[3]);
-                                system(cmd);
+                                if (system(cmd));
                                 cpl_free(cmd);
                                 cmd = cpl_sprintf("echo '%12.6g' >>"
                                                   "$KMCLIPM_DEBUG/tmp/kmclipm_offset.txt",
                                                   fitted_pars[4]);
-                                system(cmd);
+                                if (system(cmd));
                                 cpl_free(cmd);
                             }
                         } else {
@@ -1052,7 +1052,7 @@ cpl_error_code kmclipm_rtd_image_from_memory(const cpl_image *actual_img,
                                 char *cmd = cpl_sprintf("echo '%d : fit failed, returning -1' >>"
                                                         "$KMCLIPM_DEBUG/tmp/kmclipm_fitpar.txt",
                                                         ifu_nr);
-                                system(cmd);
+                                if (system(cmd));
                                 cpl_free(cmd);
                             }
                         }
