@@ -96,8 +96,10 @@
 #define NOISE_SPEC               "NOISE_SPEC"
 // kmo_sci_red
 #define SCIENCE                  "SCIENCE"              // RAW
+#define COMBINED_RECONS          "COMBINE_SCI_RECONSTRUCTED"
 #define COMBINED_CUBE            "SCI_COMBINED"
 #define COMBINED_COLLAPSED       "SCI_COMBINED_COLL"
+#define COMBINED_RECONS_COLLAPSED "COMBINE_SCI_RECONSTRUCTED_COLL"
 #define RECONSTRUCTED_CUBE       "SCI_RECONSTRUCTED"
 #define RECONSTRUCTED_COLLAPSED  "SCI_RECONSTRUCTED_COLL"
 #define INTERIM_OBJECT_CUBE      "SCI_INTERIM_OBJECT"
