@@ -191,7 +191,8 @@ cpl_error_code  kmo_reconstruct_sci(
                                 cpl_imagelist  **noiseCubePtr,
                                 int            flux,
                                 int            background,
-                                int            xcal_interpolation);
+                                int            xcal_interpolation,
+                                int            oscan_corr);
 
 cpl_image *kmo_calc_mode_for_flux_image(
                                 const cpl_image *data,
