@@ -32,6 +32,8 @@
 #include <cpl.h>
 
 #include <kmo_utils.h>
+#include <kmo_priv_make_image.h>
+#include <kmo_priv_functions.h>
 #include <kmo_error.h>
 #include <kmo_debug.h>
 #include <kmo_dfs.h>
@@ -46,6 +48,14 @@
  */
 /*----------------------------------------------------------------------------*/
 
+
+
+/*-----------------------------------------------------------------------------
+ *                          Functions prototypes
+ *----------------------------------------------------------------------------*/
+
+static int kmos_sci_red_clean_plist(cpl_propertylist *) ;
+
 /*----------------------------------------------------------------------------*/
 /**
   @brief    Get the pipeline copyright and license
@@ -77,6 +87,281 @@ const char * kmos_get_license(void)
     return kmos_license ;
 }
 
+/*----------------------------------------------------------------------------*/
+/**
+  @brief    Compute REFLEX SUFFIX keyword
+  @param    
+  @return  
+ */
+/*----------------------------------------------------------------------------*/
+char * kmos_get_reflex_suffix(
+        int             mapping_id,
+        const char  *   ifus_txt,
+        const char  *   name,
+        const char  *   obj_name)
+{
+    char    *   out ;
+    int         i ;
+
+    /* Check Inputs */
+    if (ifus_txt == NULL || name == NULL) return NULL;
+
+    /* Mapping */
+    if (mapping_id > 0)     return cpl_strdup("mapping") ;
+
+    /* Name is used */
+    if (strcmp(name, ""))   return cpl_strdup(name) ;
+
+    /* IFUs option is used */
+    if (strcmp(ifus_txt, "")) {
+        out = cpl_strdup(ifus_txt) ;
+
+        /* Replace ; by _ */
+        int i=0;
+        while(out[i]!='\0') {
+            if(out[i]==';') out[i]='_';
+            i++; 
+        }
+        return out ;
+    }
+
+    /* Default case */
+    return cpl_strdup(obj_name) ;
+}
+ 
+/*----------------------------------------------------------------------------*/
+/**
+  @brief    Collapse some cubes
+  @param    frameset            Set of frames 
+  @return   0 if ok, -1 in error case
+ */
+/*----------------------------------------------------------------------------*/
+int kmos_collapse_cubes(
+        char                    *   pro_catg,
+        cpl_frameset            *   frameset,
+        cpl_parameterlist       *   parlist,
+        double                      threshold,
+        const char              *   range,
+        const char              *   cmethod,
+        double                      cpos_rej,
+        double                      cneg_rej,
+        int                         citer,
+        int                         cmin,
+        int                         cmax)
+{
+    cpl_frame           *   in_cube_frame ;
+    cpl_frame           *   ref_spectrum_frame ;
+    const char          *   fn_cube ;
+    const char          *   pro_catg_out ;
+    cpl_vector          *   ranges ;
+    cpl_vector          *   spec_data_in ;
+    cpl_vector          *   spec_lambda_in ;
+    cpl_vector          *   identified_slices ;
+    cpl_propertylist    *   sub_header_data ;
+    cpl_propertylist    *   sub_header_noise ;
+    cpl_propertylist    *   plist ;
+    cpl_propertylist    *   pro_plist ;
+    cpl_imagelist       *   data_in ;
+    cpl_imagelist       *   noise_in ;
+    cpl_image           *   data_out ;
+    cpl_image           *   noise_out ;
+    kmclipm_vector      *   kmclipm_tmp_vec ;
+    double                  spec_crpix, spec_crval, spec_cdelt,
+                            ifu_crpix, ifu_crval, ifu_cdelt ;
+    char                *   filename ;
+    int                     nr_devices, devnr, index_data, index_noise ;
+    main_fits_desc          desc1 ;
+    int                     i;
+
+    /* Check Inputs */
+    if (frameset == NULL || parlist == NULL || pro_catg == NULL) return -1 ;
+
+    if (!strcmp(pro_catg, COMBINED_CUBE)) {
+        pro_catg_out = COMBINED_COLLAPSED ;
+    } else if (!strcmp(pro_catg, RECONSTRUCTED_CUBE)) {
+        pro_catg_out = RECONSTRUCTED_COLLAPSED ;
+    } else if (!strcmp(pro_catg, COMBINED_RECONS)) {
+        pro_catg_out = COMBINED_RECONS_COLLAPSED ;
+    } else {
+        return -1 ;
+    }
+
+    /* Get ranges */
+    ranges = kmo_identify_ranges(range);
+
+    /* Get OH_SPEC */
+    if (ref_spectrum_frame = kmo_dfs_get_frame(frameset, OH_SPEC)) {
+        plist = kmos_dfs_load_sub_header(ref_spectrum_frame, 1, FALSE);
+        spec_crpix = cpl_propertylist_get_double(plist, CRPIX1);
+        spec_crval = cpl_propertylist_get_double(plist, CRVAL1);
+        spec_cdelt = cpl_propertylist_get_double(plist, CDELT1);
+        cpl_propertylist_delete(plist) ;
+
+        /* Load OH lines data */
+        kmclipm_tmp_vec = kmos_dfs_load_vector(ref_spectrum_frame, 1, FALSE) ;
+        spec_data_in = kmclipm_vector_create_non_rejected(kmclipm_tmp_vec);
+        kmclipm_vector_delete(kmclipm_tmp_vec);
+
+        /* Convert threshold from percentage to absolute value */
+        threshold = threshold * cpl_vector_get_max(spec_data_in);
+
+        /* Create lambda-vector for OH-lines */
+        spec_lambda_in = kmo_create_lambda_vec(
+                cpl_vector_get_size(spec_data_in), (int)spec_crpix, spec_crval,
+                spec_cdelt) ;
+    } else {
+        spec_data_in = NULL ;
+        spec_lambda_in = NULL ;
+    }
+
+    /* Loop on the cubes with the requested PRO.CATG */
+    in_cube_frame = kmo_dfs_get_frame(frameset, pro_catg);
+    while (in_cube_frame != NULL ) {
+        fn_cube = cpl_frame_get_filename(in_cube_frame);
+        kmo_init_fits_desc(&desc1);
+        desc1 = kmo_identify_fits_header(fn_cube);
+
+        /* Create the Header and save the initial file */
+        pro_plist = cpl_propertylist_new() ;
+        plist = cpl_propertylist_load(fn_cube, 0) ;
+        cpl_propertylist_update_string(pro_plist,CPL_DFS_PRO_CATG,pro_catg_out);
+        cpl_propertylist_copy_property(pro_plist, plist, 
+                "ESO PRO REFLEX SUFFIX"); 
+        filename = cpl_sprintf("make_image_%s", fn_cube) ;
+        cpl_dfs_save_propertylist(frameset, NULL, parlist, frameset,
+                in_cube_frame, "kmos_sci_red", pro_plist, NULL, VERSION,
+                filename) ;
+        cpl_propertylist_delete(pro_plist) ;
+        cpl_propertylist_delete(plist) ;
+
+        if (desc1.ex_noise == TRUE)     nr_devices = desc1.nr_ext / 2;
+        else                            nr_devices = desc1.nr_ext;
+        for (i = 1; i <= nr_devices; i++) {
+            if (desc1.ex_noise == FALSE) devnr=desc1.sub_desc[i-1].device_nr;
+            else                         devnr=desc1.sub_desc[2*i-1].device_nr;
+            if (desc1.ex_badpix == FALSE)
+                index_data = kmo_identify_index_desc(desc1, devnr, FALSE);
+            else
+                index_data = kmo_identify_index_desc(desc1, devnr, 2);
+            index_noise = 0 ;
+            if (desc1.ex_noise)
+                index_noise = kmo_identify_index_desc(desc1, devnr, TRUE);
+
+            /* Load the Extension Header */
+            sub_header_data = kmos_dfs_load_sub_header(in_cube_frame, devnr,
+                    FALSE);
+
+            /* Load noise */
+            sub_header_noise = NULL ;
+            if (desc1.ex_noise)
+               sub_header_noise=kmos_dfs_load_sub_header(in_cube_frame, devnr,
+                       TRUE);
+
+            /* If Data Valid */
+            if (desc1.sub_desc[index_data-1].valid_data == TRUE) {
+                /* Interpolate oh-lines to fit input data */
+                ifu_crpix = cpl_propertylist_get_double(sub_header_data,CRPIX3);
+                ifu_crval = cpl_propertylist_get_double(sub_header_data,CRVAL3);
+                ifu_cdelt = cpl_propertylist_get_double(sub_header_data,CDELT3);
+
+                if (spec_data_in == NULL) {
+                    identified_slices = kmo_identify_slices(ranges, ifu_crpix,
+                            ifu_crval, ifu_cdelt, desc1.naxis3);
+                } else {
+                    identified_slices = kmo_identify_slices_with_oh(
+                            spec_data_in, spec_lambda_in, ranges, threshold,
+                            ifu_crpix, ifu_crval, ifu_cdelt, desc1.naxis3);
+                }
+                kmos_sci_red_clean_plist(sub_header_data) ;
+
+                /* Load data */
+                data_in = kmos_dfs_load_cube(in_cube_frame, devnr, FALSE) ;
+
+                /* Load noise, if existing */
+                noise_in = NULL ;
+                if (desc1.ex_noise && desc1.sub_desc[index_noise-1].valid_data)
+                    noise_in = kmos_dfs_load_cube(in_cube_frame, devnr, TRUE) ;
+
+                /* Process */
+                noise_out = NULL ;
+                kmclipm_make_image(data_in, noise_in, &data_out, &noise_out,
+                        identified_slices, cmethod, cpos_rej, cneg_rej, citer,
+                        cmax, cmin);
+                cpl_imagelist_delete(data_in);
+                if (noise_in != NULL)
+                    cpl_imagelist_delete(noise_in);
+                cpl_vector_delete(identified_slices);
+
+                /* Save Data */
+                kmclipm_image_save(data_out, filename, CPL_BPP_IEEE_FLOAT,
+                        sub_header_data, CPL_IO_EXTEND, 0./0.) ;
+
+                /* Process & save noise, if existing */
+                if (desc1.ex_noise) {
+                    kmclipm_image_save(noise_out, filename, CPL_BPP_IEEE_FLOAT,
+                            sub_header_noise, CPL_IO_EXTEND, 0./0.);
+                }
+
+                /* Free memory */
+                cpl_image_delete(data_out);
+                if (noise_out != NULL) cpl_image_delete(noise_out);
+            } else {
+                kmos_sci_red_clean_plist(sub_header_data) ;
+
+                /* invalid IFU, just save sub_headers */
+                cpl_propertylist_save(sub_header_data, filename, CPL_IO_EXTEND);                 if (desc1.ex_noise)
+                    cpl_propertylist_save(sub_header_noise, filename,
+                            CPL_IO_EXTEND);
+            }
+            cpl_propertylist_delete(sub_header_data);
+            if (sub_header_noise != NULL)
+                cpl_propertylist_delete(sub_header_noise);
+        }
+        cpl_free(filename) ;
+        kmo_free_fits_desc(&desc1);
+
+        /* Next candidate */
+        in_cube_frame = kmo_dfs_get_frame(frameset, NULL);
+    }
+
+    if (spec_data_in != NULL) cpl_vector_delete(spec_data_in);
+    if (spec_lambda_in != NULL) cpl_vector_delete(spec_lambda_in);
+    return 0 ;
+}
+
+/*----------------------------------------------------------------------------*/
+/**
+  @brief    Clean a propertylist
+  @param    plist       The propertylist to clean
+  @return   0 if ok, -1 in error case
+ */
+/*----------------------------------------------------------------------------*/
+static int kmos_sci_red_clean_plist(cpl_propertylist * plist)
+{
+    if (cpl_propertylist_has(plist, CRPIX3))
+        cpl_propertylist_erase(plist, CRPIX3);
+    if (cpl_propertylist_has(plist, CRVAL3))
+        cpl_propertylist_erase(plist, CRVAL3);
+    if (cpl_propertylist_has(plist, CDELT3))
+        cpl_propertylist_erase(plist, CDELT3);
+    if (cpl_propertylist_has(plist, CTYPE3))
+        cpl_propertylist_erase(plist, CTYPE3);
+    if (cpl_propertylist_has(plist, CUNIT3))
+        cpl_propertylist_erase(plist, CUNIT3);
+    if (cpl_propertylist_has(plist, CD1_3))
+        cpl_propertylist_erase(plist, CD1_3);
+    if (cpl_propertylist_has(plist, CD2_3))
+        cpl_propertylist_erase(plist, CD2_3);
+    if (cpl_propertylist_has(plist, CD3_3))
+        cpl_propertylist_erase(plist, CD3_3);
+    if (cpl_propertylist_has(plist, CD3_2))
+        cpl_propertylist_erase(plist, CD3_2);
+    if (cpl_propertylist_has(plist, CD3_1))
+        cpl_propertylist_erase(plist, CD3_1);
+
+    return 0 ;
+}
+
 /*----------------------------------------------------------------------------*/
 /**
   @brief    Get the frameset for a given angle and DO_CATG
