@@ -120,6 +120,13 @@ typedef struct {
  *----------------------------------------------------------------------------*/
 
 const char * kmos_get_license(void) ;
+char * kmos_get_reflex_suffix(
+        int             mapping_id,
+        const char  *   ifus_txt,
+        const char  *   name,
+        const char  *   obj_name) ;
+int kmos_collapse_cubes(char *, cpl_frameset *, cpl_parameterlist *, double, 
+        const char *, const char *, double, double, int, int, int) ;
 cpl_frame * kmos_get_angle_frame(cpl_frameset *, int, char *) ;
 cpl_frameset * kmos_get_angle_frameset(cpl_frameset *, int, char *) ;
 int * kmos_get_angles(cpl_frameset *, int *, char *) ;
