@@ -86,7 +86,7 @@ static cpl_error_code kmos_priv_sky_tweak_get_spectra(const cpl_imagelist *,
         const cpl_imagelist *, const cpl_vector *, const cpl_image *, 
         const int, cpl_bivector **, cpl_bivector **) ;
 static cpl_bivector* kmos_priv_sky_tweak_correct_vibrational_trans(
-        cpl_bivector *, cpl_bivector *) ;
+        cpl_bivector *, cpl_bivector *, int) ;
 
 /*----------------------------------------------------------------------------*/
 /**
@@ -118,7 +118,8 @@ cpl_imagelist * kmos_priv_sky_tweak(
         const cpl_propertylist  *   header, 
         float                       min_frac, 
         int                         tbsub,
-        int                         ifu_nr) 
+        int                         ifu_nr,
+        int                         skip_last) 
 {
     cpl_imagelist   *   result ;
     cpl_image       *   mask ;
@@ -174,7 +175,7 @@ cpl_imagelist * kmos_priv_sky_tweak(
 
     /* Get the corrections factors for each sub-band */
     vscales = kmos_priv_sky_tweak_correct_vibrational_trans(obj1_spectrum,
-                sky_spectrum);
+                sky_spectrum, skip_last);
     if (ifu_nr == ifu_plot) {
         cpl_plot_bivector("set title \"SCALES\";", "w lines", "",vscales) ;
     }
@@ -235,7 +236,6 @@ cpl_imagelist * kmos_priv_sky_tweak(
             kx++;
         }
     }
-    if (cpl_error_get_code()) printf("123\n") ;
     cpl_vector_delete(lambda);
     cpl_bivector_delete(obj1_spectrum);
     cpl_bivector_delete(sky_spectrum);
@@ -589,24 +589,13 @@ static cpl_error_code kmos_priv_sky_tweak_get_spectra(
 
 static cpl_bivector * kmos_priv_sky_tweak_correct_vibrational_trans(
         cpl_bivector    *   obj_spectrum,
-        cpl_bivector    *   sky_spectrum)
+        cpl_bivector    *   sky_spectrum,
+        int                 skip_last)
 {
-    cpl_bivector *result = NULL;
-    const double lambda_boundaries[]={
-        0.780, 0.824, 0.873, 0.926, 0.964, 
-        1.014, 1.067,  1.125,  
-        1.196,  1.252, 1.289,
-        1.400, 1.472, 1.5543,
-        1.6356, 1.7253, 1.840,
-        1.9570, 2.095, 2.30, 2.460};
-    const int nr_boundaries = sizeof(lambda_boundaries) / sizeof(double);
-    const char      *   labels[] = { "","","","","",
-        "4-1 transitions", "5-2 transitions", "6-3 transitions",
-        "7-4 transitions", " 02 transitions", "8-5 transitions",
-        "2-0 transitions", "3-1 transitions", "4-2 transitions",
-        "5-3 transitions", "6-4 transitions", "7-5 transitions",
-        "8-6 transitions", "9-7 transitions", "final bit"};
-    double              scalings[nr_boundaries]; 
+    cpl_bivector    *   result ;
+    double          *   scalings ;
+    double          *   lambda_boundaries ;
+    char            **  labels ;
     const double    *   lambda ;
     const double    *   intobj ;
     const double    *   intsky ;
@@ -625,11 +614,8 @@ static cpl_bivector * kmos_priv_sky_tweak_correct_vibrational_trans(
     cpl_vector      *   tmp3_s_v ;
     cpl_vector      *   tmp4_v  ;
     double              median, stdev ;
-    int                 i, ix, il, nr_lambda, nr_transitions, nr_samples;
-
-    /* Initialise */
-    median = stdev = 0.0 ;
-    tmp4_v = NULL ;
+    int                 i, ix, il, nr_lambda, nr_transitions,
+                        nr_samples, nr_boundaries ;
 
     /* Check Entries */
     if (obj_spectrum == NULL || sky_spectrum == NULL) {
@@ -638,10 +624,54 @@ static cpl_bivector * kmos_priv_sky_tweak_correct_vibrational_trans(
         return NULL;
     }
 
+    /* Initialise */
+    median = stdev = 0.0 ;
+    tmp4_v = NULL ;
+    if (skip_last) {
+        nr_boundaries = 20 ;
+        lambda_boundaries = cpl_malloc(nr_boundaries * sizeof(double)) ;
+        labels = cpl_malloc((nr_boundaries-1) * sizeof(char*)) ;
+    } else {
+        nr_boundaries = 21 ;
+        lambda_boundaries = cpl_malloc(nr_boundaries * sizeof(double)) ;
+        labels = cpl_malloc((nr_boundaries-1) * sizeof(char*)) ;
+    }
+    lambda_boundaries[0]  = 0.780 ;     labels[0]  = "" ;
+    lambda_boundaries[1]  = 0.824 ;     labels[1]  = "" ;
+    lambda_boundaries[2]  = 0.873 ;     labels[2]  = "" ;
+    lambda_boundaries[3]  = 0.926 ;     labels[3]  = "" ;
+    lambda_boundaries[4]  = 0.964 ;     labels[4]  = "" ;
+    lambda_boundaries[5]  = 1.014 ;     labels[5]  = "4-1 transitions" ;
+    lambda_boundaries[6]  = 1.067 ;     labels[6]  = "5-2 transitions" ;
+    lambda_boundaries[7]  = 1.125 ;     labels[7]  = "6-3 transitions" ;
+    lambda_boundaries[8]  = 1.196 ;     labels[8]  = "7-4 transitions" ;
+    lambda_boundaries[9]  = 1.252 ;     labels[9]  = " 02 transitions" ;
+    lambda_boundaries[10] = 1.289 ;     labels[10] = "8-5 transitions" ;
+    lambda_boundaries[11] = 1.400 ;     labels[11] = "2-0 transitions" ;
+    lambda_boundaries[12] = 1.472 ;     labels[12] = "3-1 transitions" ;
+    lambda_boundaries[13] = 1.5543 ;    labels[13] = "4-2 transitions" ;
+    lambda_boundaries[14] = 1.6356 ;    labels[14] = "5-3 transitions" ;
+    lambda_boundaries[15] = 1.7253 ;    labels[15] = "6-4 transitions" ;
+    lambda_boundaries[16] = 1.840 ;     labels[16] = "7-5 transitions" ;
+    lambda_boundaries[17] = 1.9570 ;    labels[17] = "8-6 transitions" ;
+    lambda_boundaries[18] = 2.095 ;     labels[18] = "9-7 transitions" ;
+    if (skip_last) {
+        lambda_boundaries[19] = 2.460 ; 
+    } else {
+        lambda_boundaries[19] = 2.30 ;  labels[19] = "final bit" ;
+        lambda_boundaries[20] = 2.460 ;
+    }
+
+    /* Allocate scalings */
+    scalings = cpl_malloc(nr_boundaries * sizeof(double)) ;
+
     nr_lambda = cpl_bivector_get_size(obj_spectrum);
     if (nr_lambda != cpl_bivector_get_size(sky_spectrum)) {
         cpl_msg_error(__func__, "Illegal inputs") ;
         cpl_error_set(__func__, CPL_ERROR_ILLEGAL_INPUT) ;
+        cpl_free(scalings) ;
+        cpl_free(labels) ;
+        cpl_free(lambda_boundaries) ;
         return NULL;
     }
     lambda = cpl_vector_get_data_const(cpl_bivector_get_x_const(obj_spectrum));
@@ -656,6 +686,9 @@ static cpl_bivector * kmos_priv_sky_tweak_correct_vibrational_trans(
                 >= tol) {
             cpl_msg_error(__func__, "Illegal inputs") ;
             cpl_error_set(__func__, CPL_ERROR_ILLEGAL_INPUT) ;
+            cpl_free(scalings) ;
+            cpl_free(labels) ;
+            cpl_free(lambda_boundaries) ;
             return NULL;
         }
     }
@@ -910,6 +943,7 @@ static cpl_bivector * kmos_priv_sky_tweak_correct_vibrational_trans(
         cpl_msg_debug(__func__, "%s", dbg_message);
         dbg_message[0] = '\0';
     } // end for (ix = 0; ix<nr_boundaries-1; ix++)
+    cpl_free(labels) ;
 
     // check for outliers in the scaling factors
     int nr_valid = 0,
@@ -961,6 +995,8 @@ static cpl_bivector * kmos_priv_sky_tweak_correct_vibrational_trans(
             }
         }
     }
+    cpl_free(scalings) ;
+    cpl_free(lambda_boundaries) ;
 
     scale = polynomial_irreg_irreg_nonans(scale0_size, scale0_lambda, 
             scale0_value, nr_lambda, lambda, 1);
