@@ -1,5 +1,4 @@
-/* $Id: kmo_dfs-test.c,v 1.12 2013-10-08 11:18:19 aagudo Exp $
- *
+/*
  * This file is part of the KMOS Library
  * Copyright (C) 2007-2008 European Southern Observatory
  *
@@ -18,13 +17,6 @@
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA
  */
 
-/*
- * $Author: aagudo $
- * $Date: 2013-10-08 11:18:19 $
- * $Revision: 1.12 $
- * $Name: not supported by cvs2svn $
- */
-
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
@@ -190,7 +182,7 @@ void test_kmo_dfs_load_primary_header()
     cpl_propertylist    *pl         = NULL;
     cpl_frameset        *fs         = cpl_frameset_new();
     cpl_frame           *fr         = NULL;
-    char *my_path = cpl_sprintf("%s/../../recipes/tests/ref_data/KMOS_dummy.fits", getenv("srcdir"));
+    char *my_path = cpl_sprintf("%s/ref_data/KMOS_dummy.fits", getenv("srcdir"));
     fr = kmo_test_frame_new(my_path, "", CPL_FRAME_GROUP_RAW);
     cpl_frameset_insert(fs, fr);
 
@@ -220,7 +212,7 @@ void test_kmo_dfs_load_sub_header()
     cpl_propertylist    *pl         = NULL;
     cpl_frameset        *fs         = cpl_frameset_new();
     cpl_frame           *fr         = NULL;
-    char *my_path = cpl_sprintf("%s/../../recipes/tests/ref_data/KMOS_dummy.fits", getenv("srcdir"));
+    char *my_path = cpl_sprintf("%s/ref_data/KMOS_dummy.fits", getenv("srcdir"));
     fr = kmo_test_frame_new(my_path, "", CPL_FRAME_GROUP_RAW);
     cpl_frameset_insert(fs, fr);
 
@@ -635,7 +627,7 @@ void test_kmo_dfs_save_main_header()
         kmo_dfs_save_main_header(fs, "aga", NULL, NULL, NULL, NULL, ""));
     cpl_test_error(CPL_ERROR_NULL_INPUT);
 
-    char *my_path = cpl_sprintf("%s/../../recipes/tests/ref_data/KMOS_dummy.fits", getenv("srcdir"));
+    char *my_path = cpl_sprintf("%s/ref_data/KMOS_dummy.fits", getenv("srcdir"));
     fr = kmo_test_frame_new(my_path, "", CPL_FRAME_GROUP_RAW);
     cpl_frameset_insert(fs, fr);
 
@@ -676,7 +668,7 @@ void test_kmo_dfs_save_sub_header()
     cpl_test_error(CPL_ERROR_NULL_INPUT);
     kmo_test_verbose_on();
 
-    char *my_path = cpl_sprintf("%s/../../recipes/tests/ref_data/KMOS_dummy.fits", getenv("srcdir"));
+    char *my_path = cpl_sprintf("%s/ref_data/KMOS_dummy.fits", getenv("srcdir"));
     pl = cpl_propertylist_load(my_path, 1);
     cpl_test_eq(CPL_ERROR_NONE,
         kmo_dfs_save_sub_header("agaa", "", pl));
@@ -714,7 +706,7 @@ void test_kmo_dfs_save_vector()
     /* save real vector here (first save main header)*/
     cpl_parameterlist   *parl       = cpl_parameterlist_new();
     cpl_frameset        *fs         = cpl_frameset_new();
-    char *my_path = cpl_sprintf("%s/../../recipes/tests/ref_data/KMOS_dummy.fits", getenv("srcdir"));
+    char *my_path = cpl_sprintf("%s/ref_data/KMOS_dummy.fits", getenv("srcdir"));
     cpl_frame           *fr         = kmo_test_frame_new( my_path, "", CPL_FRAME_GROUP_RAW);
     cpl_frameset_insert(fs, fr);
     cpl_test_eq(CPL_ERROR_NONE,
@@ -771,7 +763,7 @@ void test_kmo_dfs_save_image()
     /* save real image here (first save main header) */
     cpl_parameterlist   *parl       = cpl_parameterlist_new();
     cpl_frameset        *fs         = cpl_frameset_new();
-    char *my_path = cpl_sprintf("%s/../../recipes/tests/ref_data/KMOS_dummy.fits", getenv("srcdir"));
+    char *my_path = cpl_sprintf("%s/ref_data/KMOS_dummy.fits", getenv("srcdir"));
     cpl_frame           *fr         = kmo_test_frame_new(my_path, "", CPL_FRAME_GROUP_RAW);
     cpl_frameset_insert(fs, fr);
     cpl_test_eq(CPL_ERROR_NONE,
@@ -834,7 +826,7 @@ void test_kmo_dfs_save_cube()
     /* save real cube here */
     cpl_parameterlist   *parl       = cpl_parameterlist_new();
     cpl_frameset        *fs         = cpl_frameset_new();
-    char *my_path = cpl_sprintf("%s/../../recipes/tests/ref_data/KMOS_dummy.fits", getenv("srcdir"));
+    char *my_path = cpl_sprintf("%s/ref_data/KMOS_dummy.fits", getenv("srcdir"));
     cpl_frame           *fr         = kmo_test_frame_new( my_path, "", CPL_FRAME_GROUP_RAW);
     cpl_frameset_insert(fs, fr);
     cpl_test_eq(CPL_ERROR_NONE,
@@ -903,7 +895,7 @@ void test_kmo_dfs_get_frame()
     cpl_test_error(CPL_ERROR_ILLEGAL_INPUT);
     kmo_test_verbose_on();
 
-    char *my_path = cpl_sprintf("%s/../../recipes/tests/ref_data/KMOS_dummy.fits", getenv("srcdir"));
+    char *my_path = cpl_sprintf("%s/tests/ref_data/KMOS_dummy.fits", getenv("srcdir"));
     fr = kmo_test_frame_new(my_path, "", CPL_FRAME_GROUP_RAW);
     cpl_frameset_insert(fs, fr);
     cpl_frame *ff;
@@ -1154,7 +1146,7 @@ void test_kmo_identify_index()
 {
     cpl_frameset        *fs         = cpl_frameset_new();
     cpl_frame           *fr         = NULL;
-    char *my_path = cpl_sprintf("%s/../../recipes/tests/ref_data/KMOS_dummy.fits", getenv("srcdir"));
+    char *my_path = cpl_sprintf("%s/ref_data/KMOS_dummy.fits", getenv("srcdir"));
     fr = kmo_test_frame_new(my_path, "", CPL_FRAME_GROUP_RAW);
     cpl_frameset_insert(fs, fr);
 
@@ -1211,7 +1203,7 @@ void test_kmo_check_lamp()
  */
 int main()
 {
-    cpl_test_init("<kmos-spark@mpe.mpg.de>", CPL_MSG_WARNING);
+    cpl_test_init("<usd-help@eso.org>", CPL_MSG_WARNING);
 
     test_strlower();
     test_kmo_extname_extractor();
