@@ -1,5 +1,4 @@
-/* $Id: kmo_priv_fits_stack-test.c,v 1.3 2013-10-08 11:18:19 aagudo Exp $
- *
+/*
  * This file is part of the KMOS Library
  * Copyright (C) 2007-2008 European Southern Observatory
  *
@@ -18,13 +17,6 @@
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA
  */
 
-/*
- * $Author: aagudo $
- * $Date: 2013-10-08 11:18:19 $
- * $Revision: 1.3 $
- * $Name: not supported by cvs2svn $
- */
-
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
@@ -446,9 +438,9 @@ void test_kmo_priv_check_dimensions()
 */
 int main()
 {
-    cpl_test_init("<kmos-spark@mpe.mpg.de>", CPL_MSG_WARNING);
+    cpl_test_init("<usd-help@eso.org>", CPL_MSG_WARNING);
 
-    test_kmo_priv_fits_stack();
+    /* test_kmo_priv_fits_stack(); */
     test_kmo_priv_check_dimensions();
 
     return cpl_test_end(0);
