@@ -1,5 +1,4 @@
-/* $Id: kmo_copy.c,v 1.12 2013-05-21 12:13:58 aagudo Exp $
- *
+/* 
  * This file is part of the KMOS Pipeline
  * Copyright (C) 2002,2003 European Southern Observatory
  *
@@ -18,13 +17,6 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
-/*
- * $Author: aagudo $
- * $Date: 2013-05-21 12:13:58 $
- * $Revision: 1.12 $
- * $Name: not supported by cvs2svn $
- */
-
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
@@ -157,7 +149,7 @@ int cpl_plugin_get_info(cpl_pluginlist *list)
                             "image or spectrum",
                         kmo_copy_description,
                         "Alex Agudo Berbel",
-                        "kmos-spark@mpe.mpg.de",
+                        "usd-help@eso.org",
                         kmos_get_license(),
                         kmo_copy_create,
                         kmo_copy_exec,
