@@ -1,5 +1,4 @@
-/* $Id: kmo_fits_strip.c,v 1.14 2013/08/02 09:09:54 aagudo Exp $
- *
+/*
  * This file is part of the KMOS Pipeline
  * Copyright (C) 2002,2003 European Southern Observatory
  *
@@ -18,13 +17,6 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
-/*
- * $Author: aagudo $
- * $Date: 2013/08/02 09:09:54 $
- * $Revision: 1.14 $
- * $Name:  $
- */
-
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
@@ -122,7 +114,7 @@ int cpl_plugin_get_info(cpl_pluginlist *list)
                         "processed KMOS fits frame",
                         kmo_fits_strip_description,
                         "Alex Agudo Berbel",
-                        "kmos-spark@mpe.mpg.de",
+                        "usd-help@eso.org",
                         kmos_get_license(),
                         kmo_fits_strip_create,
                         kmo_fits_strip_exec,
