@@ -1,5 +1,4 @@
-/* $Id: kmo_stats.c,v 1.16 2013-05-24 13:35:28 aagudo Exp $
- *
+/* 
  * This file is part of the KMOS Pipeline
  * Copyright (C) 2002,2003 European Southern Observatory
  *
@@ -18,13 +17,6 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
-/*
- * $Author: aagudo $
- * $Date: 2013-05-24 13:35:28 $
- * $Revision: 1.16 $
- * $Name: not supported by cvs2svn $
- */
-
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
@@ -160,7 +152,7 @@ int cpl_plugin_get_info(cpl_pluginlist *list)
                         "Perform basic statistics on a KMOS-conform fits-file",
                         kmo_stats_description,
                         "Alex Agudo Berbel",
-                        "kmos-spark@mpe.mpg.de",
+                        "usd-help@eso.org",
                         kmos_get_license(),
                         kmo_stats_create,
                         kmo_stats_exec,
@@ -205,7 +197,7 @@ static int kmo_stats_create(cpl_plugin *plugin)
     cpl_parameterlist_append(recipe->parameters, p);
 
     /* Fill the parameters list */
-    return kmo_combine_pars_create(recipe->parameters,
+    return kmos_combine_pars_create(recipe->parameters,
                                    "kmos.kmo_stats",
                                    DEF_REJ_METHOD,
                                    TRUE);
@@ -367,7 +359,7 @@ static int kmo_stats(cpl_parameterlist *parlist, cpl_frameset *frameset)
                         "extensions!");
 
         KMO_TRY_EXIT_IF_ERROR(
-            kmo_combine_pars_load(parlist,
+            kmos_combine_pars_load(parlist,
                                   "kmos.kmo_stats",
                                   &cmethod,
                                   &cpos_rej,
