@@ -44,10 +44,9 @@
  *                          Functions prototypes
  *----------------------------------------------------------------------------*/
 
-static cpl_vector * kmos_combine_parse_skipped(const char *) ;
-static cpl_frameset * kmos_combine_purge_frameset(const cpl_frameset *,
-        const cpl_vector *) ;
-static int kmos_combine_is_skipped(int, const cpl_vector *) ;
+static char * kmos_combine_create_used_ifus_string(const int *,const int *,int);
+static cpl_bivector * kmos_combine_parse_skipped(const char *) ;
+static int kmos_combine_is_skipped(const cpl_bivector *, int, int) ;
 
 static int kmos_combine_create(cpl_plugin *);
 static int kmos_combine_exec(cpl_plugin *);
@@ -100,10 +99,12 @@ static char kmos_combine_description[] =
 "  Output files:\n"
 "\n"
 "   DO                      KMOS\n"
-"   category                Type   Explanation\n"
-"   --------                -----  -----------\n"
-"   COMBINE_<ESO PRO CATG>  F3I    Combined data cube\n"
-"   EXP_MASK_<ESO PRO CATG> F3I    Exposure time mask\n"
+"   category                Type    Explanation\n"
+"   --------                -----   -----------\n"
+"   COMBINE_<ESO PRO CATG>  F3I     Combined data cube\n"
+"   EXP_MASK_<ESO PRO CATG> F3I     Exposure time mask\n"
+"   SCI_COMBINED_COLL               (optional) Collapsed combined cube\n"
+"                                   (set --collapse_combined)\n"
 "---------------------------------------------------------------------------\n"
 "\n";
 
@@ -240,8 +241,9 @@ static int kmos_combine_create(cpl_plugin *plugin)
     /* --skipped_frames */
     p = cpl_parameter_new_value("kmos.kmos_combine.skipped_frames", 
             CPL_TYPE_STRING,
-            "List of indexes of the frames to skip. n means the nth RAW "
-            "frames will be ignored", "kmos.kmos_combine", "");
+            "Comma separated List of IFUs to skip for the combination. An IFU is specified with R:I."
+            "R is the index (starting at 1) of the reconstructed frame, I the IFU number", 
+            "kmos.kmos_combine", "");
     cpl_parameter_set_alias(p, CPL_PARAMETER_MODE_CLI, "skipped_frames");
     cpl_parameter_disable(p, CPL_PARAMETER_MODE_ENV);
     cpl_parameterlist_append(recipe->parameters, p);
@@ -254,6 +256,14 @@ static int kmos_combine_create(cpl_plugin *plugin)
     cpl_parameter_disable(p, CPL_PARAMETER_MODE_ENV);
     cpl_parameterlist_append(recipe->parameters, p);
 
+    /* --collapse_combined */
+    p = cpl_parameter_new_value("kmos.kmos_combine.collapse_combined",
+            CPL_TYPE_BOOL, "Flag to collapse the combined images",
+            "kmos.kmos_combine", FALSE);
+    cpl_parameter_set_alias(p, CPL_PARAMETER_MODE_CLI, "collapse_combined");
+    cpl_parameter_disable(p, CPL_PARAMETER_MODE_ENV);
+    cpl_parameterlist_append(recipe->parameters, p);
+
     return kmos_combine_pars_create(recipe->parameters, "kmos.kmos_combine",
             DEF_REJ_METHOD, FALSE);
 }
@@ -314,6 +324,7 @@ static int kmos_combine_destroy(cpl_plugin *plugin)
 static int kmos_combine(cpl_parameterlist *parlist, cpl_frameset *frameset)
 {
     const cpl_parameter *   par ;
+    cpl_frameset        *   rawframes ;
     const char      * filename,* fmethod,* method,* ifus_txt,* cmethod,
                     * skipped_frames;
     double          cpos_rej, cneg_rej ;
@@ -321,20 +332,24 @@ static int kmos_combine(cpl_parameterlist *parlist, cpl_frameset *frameset)
     int                 citer, cmin, cmax, nr_frames, index,
                         data_cube_counter, noise_cube_counter, flux,
                         edge_nan, name_vec_size, found, suppress_extension,
-                        suppress_index, ifu_nr, nv ;
+                        suppress_index, ifu_nr, nv, collapse_combined,
+                        mapping_id ;
     char            * tmp_str, * fn_combine, * fn_mask, * mapping_mode,
-                    * name, ** name_vec ;
+                    * name, ** name_vec, * name_loc ;
     const char      * frame_filename, * tmp_strc ;
-    cpl_frameset    * frameset_clean ;
     cpl_image       * exp_mask ;
     cpl_imagelist   ** data_cube_list, ** noise_cube_list, * cube_combined_data,
                     * cube_combined_noise ;
     cpl_propertylist    * main_header, **data_header_list, **noise_header_list,
-                        * tmp_header ;
-    cpl_vector      *   skipped_vector ;
+                        * tmp_header, * pro_plist ;
+    char                *   reflex_suffix ;
+    cpl_bivector    *   skipped_bivector ;
     cpl_frame       *   frame ;
     cpl_size            ci ;
     main_fits_desc      desc;
+    int             *   used_frame_idx ;
+    int             *   used_ifus ;
+    char            *   used_ifus_str ;
     enum extrapolationType  extrapol_enum = NONE_CLIPPING;
     int             i, j ;
 
@@ -368,6 +383,9 @@ static int kmos_combine(cpl_parameterlist *parlist, cpl_frameset *frameset)
     suppress_extension = cpl_parameter_get_bool(par) ;
     kmos_combine_pars_load(parlist, "kmos.kmos_combine", &cmethod, &cpos_rej, 
             &cneg_rej, &citer, &cmin, &cmax, FALSE);
+    par = cpl_parameterlist_find_const(parlist,
+            "kmos.kmos_combine.collapse_combined");
+    collapse_combined = cpl_parameter_get_bool(par);
 
     /* Check Parameters */
     if (strcmp(method, "none") && strcmp(method, "header") &&
@@ -398,31 +416,23 @@ static int kmos_combine(cpl_parameterlist *parlist, cpl_frameset *frameset)
     }
 
     /* Parse the skipped frames option */
-    skipped_vector = NULL ;
+    skipped_bivector = NULL ;
     if (strcmp(skipped_frames, "")) {
-        skipped_vector = kmos_combine_parse_skipped(skipped_frames) ;
-        if (skipped_vector != NULL) 
+        skipped_bivector = kmos_combine_parse_skipped(skipped_frames) ;
+        if (skipped_bivector != NULL) 
             cpl_msg_info(__func__, "Skip the following frames: %s", 
                     skipped_frames);
     } 
 
-    /* Create frameset_clean */
-    if (skipped_vector != NULL) {
-        frameset_clean = kmos_combine_purge_frameset(frameset, skipped_vector) ;
-        cpl_vector_delete(skipped_vector) ;
-    } else {
-        frameset_clean = cpl_frameset_duplicate(frameset) ;
-    }
-    if (frameset_clean == NULL) {
-        cpl_msg_error(__func__, "The cleaned frameset is empty") ;
-        cpl_error_set(__func__, CPL_ERROR_ILLEGAL_INPUT) ;
-        return -1 ;
-    }
+    /* Get the frames to combine - Filter out OH_SPEC */
+    rawframes = cpl_frameset_duplicate(frameset) ;
+    cpl_frameset_erase(rawframes, "OH_SPEC") ;
 
     /* Check Nb of frames */
-    nr_frames = cpl_frameset_get_size(frameset_clean);
+    nr_frames = cpl_frameset_get_size(rawframes);
     if (nr_frames < 2) {
-        cpl_frameset_delete(frameset_clean) ;
+        if (skipped_bivector!=NULL) cpl_bivector_delete(skipped_bivector) ;
+        cpl_frameset_delete(rawframes) ;
         cpl_msg_error(__func__, "At least two frames must be provided") ;
         cpl_error_set(__func__, CPL_ERROR_ILLEGAL_INPUT) ;
         return -1 ;
@@ -433,7 +443,8 @@ static int kmos_combine(cpl_parameterlist *parlist, cpl_frameset *frameset)
         ifus = kmo_identify_values(ifus_txt);
         if (ifus == NULL || cpl_vector_get_size(ifus) != nr_frames) {
             if (ifus != NULL) cpl_vector_delete(ifus);
-            cpl_frameset_delete(frameset_clean) ;
+            if (skipped_bivector!=NULL) cpl_bivector_delete(skipped_bivector) ;
+            cpl_frameset_delete(rawframes) ;
             cpl_msg_error(__func__, "ifus size must match the science frames") ;
             cpl_error_set(__func__, CPL_ERROR_ILLEGAL_INPUT) ;
             return -1 ;
@@ -444,9 +455,9 @@ static int kmos_combine(cpl_parameterlist *parlist, cpl_frameset *frameset)
 
     /* Check for mapping mode */
     mapping_mode = NULL ;
-    cpl_size fs_size = cpl_frameset_get_size(frameset_clean);
+    cpl_size fs_size = cpl_frameset_get_size(rawframes);
     for (ci = 0; ci < fs_size; ci++) {
-        frame = cpl_frameset_get_position(frameset_clean, ci);
+        frame = cpl_frameset_get_position(rawframes, ci);
         tmp_header = kmclipm_propertylist_load(cpl_frame_get_filename(frame),0);
         if (cpl_propertylist_has(tmp_header, TPL_ID)) {
             tmp_strc = cpl_propertylist_get_string(tmp_header, TPL_ID);
@@ -479,6 +490,15 @@ static int kmos_combine(cpl_parameterlist *parlist, cpl_frameset *frameset)
         }
     }
 
+    /* Mapping Id */
+    mapping_id = -1 ;
+    if (mapping_mode == NULL) {
+        mapping_id = 0 ;
+    } else {
+        if (!strcmp(mapping_mode, MAPPING8)) mapping_id = 1 ;
+        if (!strcmp(mapping_mode, MAPPING24)) mapping_id = 2 ;
+    }
+
     /* Create name/ifu map... */
     name_vec = cpl_calloc(nr_frames*KMOS_NR_IFUS, sizeof(char*));
 
@@ -488,7 +508,7 @@ static int kmos_combine(cpl_parameterlist *parlist, cpl_frameset *frameset)
         name_vec_size = 0;
         for (i = 0; i < nr_frames; i++) {
             tmp_str = cpl_sprintf("%d", i);
-            frame = kmo_dfs_get_frame(frameset_clean, tmp_str);
+            frame = kmo_dfs_get_frame(rawframes, tmp_str);
             cpl_free(tmp_str);
             for (ifu_nr = 1; ifu_nr <= KMOS_NR_IFUS; ifu_nr++) {
                 found = 0;
@@ -549,6 +569,7 @@ static int kmos_combine(cpl_parameterlist *parlist, cpl_frameset *frameset)
  
     /*
     if (ifus != NULL) cpl_vector_delete(ifus);
+    cpl_frameset_delete(rawframes) ;
     cpl_free(data_cube_list);
     cpl_free(noise_cube_list);
     cpl_free(data_header_list);
@@ -556,19 +577,21 @@ static int kmos_combine(cpl_parameterlist *parlist, cpl_frameset *frameset)
     for (i = 0; i < name_vec_size ; i++) cpl_free(name_vec[i]);
     cpl_free(name_vec);
     if (mapping_mode != NULL) cpl_free(mapping_mode) ;
-    cpl_frameset_delete(frameset_clean) ;
     return 0 ;
     */
 
     /* Load all data (and noise if existent) cubes and store them */
     for (nv = 0; nv < name_vec_size; nv++) {
-        name = name_vec[nv];
+        name_loc = name_vec[nv];
 
+        used_frame_idx = cpl_calloc(nr_frames*KMOS_NR_IFUS, sizeof(int)) ;
+        used_ifus = cpl_calloc(nr_frames*KMOS_NR_IFUS, sizeof(int)) ;
+        
         data_cube_counter = 0;
         noise_cube_counter = 0;
         for (i = 0; i < nr_frames; i++) {
             tmp_str = cpl_sprintf("%d", i);
-            frame = kmo_dfs_get_frame(frameset_clean, tmp_str);
+            frame = kmo_dfs_get_frame(rawframes, tmp_str);
             frame_filename = cpl_frame_get_filename(frame);
             kmo_init_fits_desc(&desc);
             desc = kmo_identify_fits_header(frame_filename);
@@ -577,12 +600,12 @@ static int kmos_combine(cpl_parameterlist *parlist, cpl_frameset *frameset)
                 /* Mapping mode */
                 for (j = 1; j <= KMOS_NR_IFUS; j++) {
                     /* Loop over all IFUs */
-                    if (desc.sub_desc[j-1].valid_data == TRUE) {
-                        
+                    if (desc.sub_desc[j-1].valid_data == TRUE &&
+                            !kmos_combine_is_skipped(skipped_bivector,i+1,j)) {
                         /* Load data frames */
                         override_err_msg = TRUE;
                         data_cube_list[data_cube_counter] =
-                            kmo_dfs_load_cube(frameset_clean, tmp_str, j,FALSE);
+                            kmo_dfs_load_cube(rawframes, tmp_str, j,FALSE);
                         override_err_msg = FALSE;
                         if (data_cube_list[data_cube_counter] == NULL) {
                             cpl_error_reset();
@@ -590,7 +613,7 @@ static int kmos_combine(cpl_parameterlist *parlist, cpl_frameset *frameset)
                             if (edge_nan) kmo_edge_nan(
                                     data_cube_list[data_cube_counter], j);
                             data_header_list[data_cube_counter] =
-                                kmo_dfs_load_sub_header(frameset_clean, 
+                                kmo_dfs_load_sub_header(rawframes, 
                                         tmp_str, j, FALSE);
                             cpl_propertylist_update_string(
                                     data_header_list[data_cube_counter],
@@ -598,13 +621,15 @@ static int kmos_combine(cpl_parameterlist *parlist, cpl_frameset *frameset)
                             cpl_propertylist_update_int(
                                     data_header_list[data_cube_counter],
                                     "ESO PRO IFUNR", j);
+                            used_frame_idx[data_cube_counter] = i+1 ;
+                            used_ifus[data_cube_counter] = j ;
                             data_cube_counter++;
                         }
 
                         /* Load noise frames */
                         override_err_msg = TRUE;
                         noise_cube_list[noise_cube_counter] =
-                            kmo_dfs_load_cube(frameset_clean, tmp_str, j, TRUE);
+                            kmo_dfs_load_cube(rawframes, tmp_str, j, TRUE);
 
                         override_err_msg = FALSE;
                         if (noise_cube_list[noise_cube_counter] == NULL) {
@@ -614,7 +639,7 @@ static int kmos_combine(cpl_parameterlist *parlist, cpl_frameset *frameset)
                             if (edge_nan) kmo_edge_nan(
                                     noise_cube_list[noise_cube_counter], j);
                             noise_header_list[noise_cube_counter] =
-                                kmo_dfs_load_sub_header(frameset_clean, tmp_str,
+                                kmo_dfs_load_sub_header(rawframes, tmp_str,
                                         j, TRUE);
                             noise_cube_counter++;
                         }
@@ -633,15 +658,16 @@ static int kmos_combine(cpl_parameterlist *parlist, cpl_frameset *frameset)
             } else {
                 /* name/ifu mode (single) */
                 if (ifus != NULL)   ifu_nr = cpl_vector_get(ifus, i);
-                else        ifu_nr = kmo_get_index_from_ocs_name(frame, name);
+                else ifu_nr = kmo_get_index_from_ocs_name(frame, name_loc);
                 if (ifu_nr > 0) {
                     index = kmo_identify_index(frame_filename, ifu_nr , FALSE);
-
-                    if (desc.sub_desc[index-1].valid_data == TRUE) {
+                    if (desc.sub_desc[index-1].valid_data == TRUE &&
+                            !kmos_combine_is_skipped(skipped_bivector,i+1,
+                                ifu_nr)) {
                         /* Load data frames */
                         override_err_msg = TRUE;
                         data_cube_list[data_cube_counter] =
-                            kmo_dfs_load_cube(frameset_clean, tmp_str, ifu_nr, 
+                            kmo_dfs_load_cube(rawframes, tmp_str, ifu_nr, 
                                     FALSE);
                         override_err_msg = FALSE;
                         if (data_cube_list[data_cube_counter] == NULL) {
@@ -652,12 +678,12 @@ static int kmos_combine(cpl_parameterlist *parlist, cpl_frameset *frameset)
                                     ifu_nr, frame_filename);
                             else                cpl_msg_warning(cpl_func, 
                                     "Object %s missing in Frame %d (%s)",
-                                    name,  i+1, frame_filename) ;
+                                    name_loc,  i+1, frame_filename) ;
                         } else {
                             if (edge_nan) kmo_edge_nan(
                                     data_cube_list[data_cube_counter], ifu_nr);
                             data_header_list[data_cube_counter] =
-                                kmo_dfs_load_sub_header(frameset_clean, tmp_str,
+                                kmo_dfs_load_sub_header(rawframes, tmp_str,
                                         ifu_nr, FALSE);
                             cpl_propertylist_update_string(
                                     data_header_list[data_cube_counter],
@@ -665,13 +691,15 @@ static int kmos_combine(cpl_parameterlist *parlist, cpl_frameset *frameset)
                             cpl_propertylist_update_int(
                                     data_header_list[data_cube_counter],
                                     "ESO PRO IFUNR", ifu_nr);
+                            used_frame_idx[data_cube_counter] = i+1 ;
+                            used_ifus[data_cube_counter] = ifu_nr ;
                             data_cube_counter++;
                         }
 
                         /* Load noise frames */
                         override_err_msg = TRUE;
                         noise_cube_list[noise_cube_counter] =
-                            kmo_dfs_load_cube(frameset_clean, tmp_str, ifu_nr, 
+                            kmo_dfs_load_cube(rawframes, tmp_str, ifu_nr, 
                                     TRUE);
                         override_err_msg = FALSE;
                         if (noise_cube_list[noise_cube_counter] == NULL) {
@@ -681,7 +709,7 @@ static int kmos_combine(cpl_parameterlist *parlist, cpl_frameset *frameset)
                             if (edge_nan) kmo_edge_nan(
                                     noise_cube_list[noise_cube_counter],ifu_nr);
                             noise_header_list[noise_cube_counter] =
-                                    kmo_dfs_load_sub_header(frameset_clean, 
+                                    kmo_dfs_load_sub_header(rawframes, 
                                             tmp_str, ifu_nr, TRUE);
                             noise_cube_counter++;
                         }
@@ -689,6 +717,7 @@ static int kmos_combine(cpl_parameterlist *parlist, cpl_frameset *frameset)
                         /* Check if number of data and noise frames match */
                         if (noise_cube_counter > 0) {
                             if (data_cube_counter != noise_cube_counter) {
+                                /* TODO - deallocate */
                                 cpl_msg_error(__func__, "Noise missing") ;
                                 cpl_error_set(__func__,CPL_ERROR_ILLEGAL_INPUT);
                                 return -1 ;
@@ -701,16 +730,22 @@ static int kmos_combine(cpl_parameterlist *parlist, cpl_frameset *frameset)
             cpl_free(tmp_str);
         } 
  
+        /* Create String for the output header - IFUs usage */
+        used_ifus_str = kmos_combine_create_used_ifus_string(used_frame_idx, 
+                used_ifus, data_cube_counter);
+        cpl_free(used_frame_idx) ;
+        cpl_free(used_ifus) ;
+
         /* Combine data */
         exp_mask = NULL ;
         cube_combined_noise = NULL ;
         cube_combined_data = NULL ;
         if (kmo_priv_combine(data_cube_list, noise_cube_list, data_header_list,
                     noise_header_list, data_cube_counter, noise_cube_counter, 
-                    name, ifus_txt, method, "BCS", fmethod, filename, cmethod, 
-                    cpos_rej, cneg_rej, citer, cmin, cmax, extrapol_enum, flux,
-                    &cube_combined_data, &cube_combined_noise, 
-                    &exp_mask) != CPL_ERROR_NONE) {
+                    name_loc, ifus_txt, method, "BCS", fmethod, filename, 
+                    cmethod, cpos_rej, cneg_rej, citer, cmin, cmax, 
+                    extrapol_enum, flux, &cube_combined_data, 
+                    &cube_combined_noise, &exp_mask) != CPL_ERROR_NONE) {
             for (i = 0; i < data_cube_counter ; i++) 
                 cpl_imagelist_delete(data_cube_list[i]);
             for (i = 0; i < noise_cube_counter ; i++) 
@@ -726,6 +761,8 @@ static int kmos_combine(cpl_parameterlist *parlist, cpl_frameset *frameset)
             cpl_free(noise_header_list);
             for (i = 0; i < name_vec_size ; i++) cpl_free(name_vec[i]);
             cpl_free(name_vec);
+            cpl_free(used_ifus_str) ;
+            cpl_frameset_delete(rawframes) ;
             if (mapping_mode != NULL) cpl_free(mapping_mode) ;
             cpl_msg_error(__func__, "Failed Combination") ;
             cpl_error_set(__func__,CPL_ERROR_ILLEGAL_INPUT);
@@ -737,7 +774,6 @@ static int kmos_combine(cpl_parameterlist *parlist, cpl_frameset *frameset)
             cpl_imagelist_delete(noise_cube_list[i]);
         
         /*
-        cpl_frameset_delete(frameset_clean) ;
         for (i = 0; i < data_cube_counter ; i++) 
             cpl_propertylist_delete(data_header_list[i]);
         for (i = 0; i < noise_cube_counter ; i++) 
@@ -755,11 +791,12 @@ static int kmos_combine(cpl_parameterlist *parlist, cpl_frameset *frameset)
         cpl_imagelist_delete(cube_combined_data);
         if (noise_header_list!=NULL && noise_cube_counter==0) 
             cpl_propertylist_delete(noise_header_list[0]) ;
+        cpl_frameset_delete(rawframes) ;
         */
         /* Save data */
         if (!suppress_extension) {
             /* Setup output category COMBINE + ESO PRO CATG */
-            main_header = kmo_dfs_load_primary_header(frameset_clean, "0");
+            main_header = kmo_dfs_load_primary_header(rawframes, "0");
             fn_combine = cpl_sprintf("%s_%s_%s", COMBINE,
                     cpl_propertylist_get_string(main_header, CPL_DFS_PRO_CATG),
                     name_vec[nv]);
@@ -772,11 +809,31 @@ static int kmos_combine(cpl_parameterlist *parlist, cpl_frameset *frameset)
             fn_mask = cpl_sprintf("%s_%d", EXP_MASK, suppress_index++);
         }
 
-        frame = cpl_frameset_find(frameset, NULL);
-        kmo_dfs_save_main_header(frameset, fn_combine, "", frame, NULL, parlist,
-                cpl_func);
-        kmo_dfs_save_main_header(frameset, fn_mask, "", frame, NULL, parlist,
-                cpl_func);
+        /* Create PRO keys plist */
+        pro_plist = cpl_propertylist_new() ;
+        if (used_ifus_str != NULL) 
+            cpl_propertylist_update_string(pro_plist, "ESO PRO USEDIFUS", 
+                    used_ifus_str) ;
+        cpl_propertylist_update_string(pro_plist, "ESO PRO SKIPPEDIFUS", 
+                skipped_frames) ;
+
+        /* Add REFLEX SUFFIX keyword */
+        reflex_suffix = kmos_get_reflex_suffix(mapping_id,
+                ifus_txt, name, name_loc) ;
+        cpl_propertylist_update_string(pro_plist, "ESO PRO REFLEX SUFFIX", 
+                reflex_suffix) ;
+        cpl_free(reflex_suffix) ;
+
+        /* Save Headers first */
+        frame = cpl_frameset_find(rawframes, NULL);
+        kmo_dfs_save_main_header(frameset, fn_combine, "", frame, pro_plist, 
+                parlist, cpl_func);
+        kmo_dfs_save_main_header(frameset, fn_mask, "", frame, pro_plist, 
+                parlist, cpl_func);
+        cpl_propertylist_delete(pro_plist) ;
+        cpl_free(used_ifus_str) ;
+
+        /* Clean */
         if (data_header_list[0] != NULL) {
             if (cpl_propertylist_has(data_header_list[0], "ESO PRO FRNAME")) {
                 cpl_propertylist_erase(data_header_list[0], "ESO PRO FRNAME");
@@ -785,6 +842,14 @@ static int kmos_combine(cpl_parameterlist *parlist, cpl_frameset *frameset)
                 cpl_propertylist_erase(data_header_list[0], "ESO PRO IFUNR");
             }
         }
+        if (noise_header_list[0] != NULL) {
+            if (cpl_propertylist_has(noise_header_list[0], "ESO PRO FRNAME")) {
+                cpl_propertylist_erase(noise_header_list[0], "ESO PRO FRNAME");
+            }
+            if (cpl_propertylist_has(noise_header_list[0], "ESO PRO IFUNR")) {
+                cpl_propertylist_erase(noise_header_list[0], "ESO PRO IFUNR");
+            }
+        }
         kmo_dfs_save_cube(cube_combined_data, fn_combine, "", 
                 data_header_list[0], 0./0.);
         cpl_imagelist_delete(cube_combined_data);
@@ -814,7 +879,8 @@ static int kmos_combine(cpl_parameterlist *parlist, cpl_frameset *frameset)
         if (noise_header_list!=NULL && noise_cube_counter==0) 
             cpl_propertylist_delete(noise_header_list[0]) ;
     } 
-    cpl_frameset_delete(frameset_clean) ;
+    cpl_frameset_delete(rawframes) ;
+    if (skipped_bivector!=NULL) cpl_bivector_delete(skipped_bivector) ;
     if (ifus != NULL) cpl_vector_delete(ifus);
     cpl_free(data_cube_list);
     cpl_free(noise_cube_list);
@@ -823,6 +889,13 @@ static int kmos_combine(cpl_parameterlist *parlist, cpl_frameset *frameset)
     for (i = 0; i < name_vec_size ; i++) cpl_free(name_vec[i]);
     cpl_free(name_vec);
     cpl_free(mapping_mode) ;
+
+    /* Collapse the combined cubes if requested */
+    if (collapse_combined) {
+        kmos_collapse_cubes(COMBINED_RECONS, frameset, parlist, 0.1, "",
+                DEF_REJ_METHOD, DEF_POS_REJ_THRES, DEF_NEG_REJ_THRES,
+                DEF_ITERATIONS, DEF_NR_MIN_REJ, DEF_NR_MAX_REJ) ;
+    }
     return 0;
 }
 
@@ -835,15 +908,59 @@ static int kmos_combine(cpl_parameterlist *parlist, cpl_frameset *frameset)
   @return 
  */
 /*----------------------------------------------------------------------------*/
-static cpl_vector * kmos_combine_parse_skipped(const char * str)
+static char * kmos_combine_create_used_ifus_string(
+        const int   *   used_frame_idx,
+        const int   *   used_ifus,
+        int             nb)
 {
-    cpl_vector  *   out ;
-    char        *   my_str ;
-    int             nb_values ;
-    char        *   s1 ;
-    char        *   s2 ;
-    int             val, ret ;
-    char            ch ;
+    char        out[8192] ;
+    char        tmp[7] ;
+    int         i ;
+
+    if (nb > 1000) return NULL ;
+    for (i=0 ; i<nb ; i++) {
+        /* Build the current string */
+        if (used_frame_idx[i] < 10 && used_ifus[i] < 10) 
+            if (i==0) sprintf(tmp, "%1d:%1d", used_frame_idx[i], used_ifus[i]);
+            else sprintf(tmp, ",%1d:%1d", used_frame_idx[i], used_ifus[i]);
+        else if (used_frame_idx[i] < 10 && used_ifus[i] >= 10) 
+            if (i==0) sprintf(tmp, "%1d:%2d", used_frame_idx[i], used_ifus[i]);
+            else sprintf(tmp, ",%1d:%2d", used_frame_idx[i], used_ifus[i]);
+        else if (used_frame_idx[i] >= 10 && used_ifus[i] < 10) 
+            if (i==0) sprintf(tmp, "%2d:%1d", used_frame_idx[i], used_ifus[i]);
+            else sprintf(tmp, ",%2d:%1d", used_frame_idx[i], used_ifus[i]);
+        else if (used_frame_idx[i] >= 10 && used_ifus[i] >= 10) 
+            if (i==0) sprintf(tmp, "%2d:%2d", used_frame_idx[i], used_ifus[i]);
+            else sprintf(tmp, ",%2d:%2d", used_frame_idx[i], used_ifus[i]);
+        else return NULL ;
+
+        if (i==0)   strcpy(out, tmp) ;
+        else        strcat(out, tmp);
+    }
+
+    /* Warning : If larger than 51 char, it does not fit in the header card */
+    if (strlen(out) > 51) return NULL ;
+
+    return cpl_strdup(out) ;
+}
+
+/*----------------------------------------------------------------------------*/
+/**
+  @brief 
+  @param  
+  @return 
+ */
+/*----------------------------------------------------------------------------*/
+static cpl_bivector * kmos_combine_parse_skipped(const char * str)
+{
+    cpl_bivector    *   out ;
+    cpl_vector      *   out_x ;
+    cpl_vector      *   out_y ;
+    char            *   my_str ;
+    int                 nb_values ;
+    char            *   s1 ;
+    char            *   s2 ;
+    int                 val1, val2, ret ;
     
     /* Check Entries */
     if (str == NULL) return NULL ;
@@ -857,16 +974,19 @@ static cpl_vector * kmos_combine_parse_skipped(const char * str)
         while (*s2 == ' ' || *s2 == '\t') s2++;
         s1 = strsep(&s2, ",") ;
         if (*s1) {
-            if (sscanf(s1," %i %c", &val, &ch) == 1) {
+            if (sscanf(s1," %i:%i", &val1, &val2) == 2) {
                 nb_values ++ ;
             }
         }
     }
     cpl_free(my_str) ;
     if (nb_values == 0) return NULL ;
-    /* Create the vector */
-    out = cpl_vector_new(nb_values) ;
 
+    /* Create the vector */
+    out = cpl_bivector_new(nb_values) ;
+    out_x = cpl_bivector_get_x(out) ;
+    out_y = cpl_bivector_get_y(out) ;
+    
     /* Fill the vector */
     nb_values = 0 ;
     my_str = cpl_strdup(str) ;
@@ -874,8 +994,9 @@ static cpl_vector * kmos_combine_parse_skipped(const char * str)
         while (*s2 == ' ' || *s2 == '\t') s2++;
         s1 = strsep(&s2, ",") ;
         if (*s1) {
-            if (sscanf(s1," %i %c", &val, &ch) == 1) {
-                cpl_vector_set(out, nb_values, val) ;
+            if (sscanf(s1," %i:%i", &val1, &val2) == 2) {
+                cpl_vector_set(out_x, nb_values, val1) ;
+                cpl_vector_set(out_y, nb_values, val2) ;
                 nb_values ++ ;
             }
         }
@@ -886,63 +1007,35 @@ static cpl_vector * kmos_combine_parse_skipped(const char * str)
 
 /*----------------------------------------------------------------------------*/
 /**
-  @brief 
-  @param  
-  @return 
+  @brief    Tells if an IFU is skipped or not
+  @param    The skipped bivector
+  @param    frame index (1->first raw file)
+  @param    The IFU number
+  @return   Boolean : 1 if the IFU needs to be skipped.
  */
 /*----------------------------------------------------------------------------*/
-static cpl_frameset * kmos_combine_purge_frameset(
-        const cpl_frameset  *   in,
-        const cpl_vector    *   skipped)
+static int kmos_combine_is_skipped(
+        const cpl_bivector  *   skipped, 
+        int                     frame_idx,
+        int                     ifu_nr) 
 {
-    cpl_frameset            *   out ;
-    const cpl_frame         *   frame ;
-    int                         raw_n ;
-    cpl_frameset_iterator   *   it;
-
-    /* Check Entries */
-    if (in == NULL) return NULL ;
-
-    /* Initialise */
-    raw_n = 0 ;
-    out = cpl_frameset_new(); 
-
-    it = cpl_frameset_iterator_new(in);
-    while ((frame = cpl_frameset_iterator_get(it)) != NULL) {
-        if (cpl_frame_get_group(frame) == CPL_FRAME_GROUP_RAW) {
-            raw_n ++ ;
-            if (!kmos_combine_is_skipped(raw_n, skipped)) {
-                cpl_frameset_insert(out, cpl_frame_duplicate(frame)) ;
-            }
-        } else {
-            cpl_frameset_insert(out, cpl_frame_duplicate(frame)) ;
-        }
-        cpl_frameset_iterator_advance(it, 1);
-    }
-    cpl_frameset_iterator_delete(it);
-
-    return out;
-}
-
-/*----------------------------------------------------------------------------*/
-/**
-  @brief 
-  @param  
-  @return 
- */
-/*----------------------------------------------------------------------------*/
-static int kmos_combine_is_skipped(int idx, const cpl_vector * skipped) 
-{
-    double      val ;
-    int         i ;
+    const cpl_vector    *   vec_x ;
+    const cpl_vector    *   vec_y ;
+    double                  val1, val2 ;
+    int                     i ;
 
     /* Check entries */
     if (skipped == NULL) return 0;
 
+    /* Initialise */
+    vec_x = cpl_bivector_get_x_const(skipped) ;
+    vec_y = cpl_bivector_get_y_const(skipped) ;
+
     /* Loop */
-    for (i=0 ; i<cpl_vector_get_size(skipped) ; i++) {
-        val = cpl_vector_get(skipped, i) ;
-        if (fabs(val-idx) < 1e-3) return 1 ; 
+    for (i=0 ; i<cpl_bivector_get_size(skipped) ; i++) {
+        val1 = cpl_vector_get(vec_x, i) ;
+        val2 = cpl_vector_get(vec_y, i) ;
+        if (fabs(val1-frame_idx)<1e-3 && fabs(val2-ifu_nr)<1e-3) return 1 ; 
     }
     return 0 ;
 }
