@@ -40,6 +40,7 @@
 #include "kmo_priv_functions.h"
 #include "kmo_cpl_extensions.h"
 #include "kmo_debug.h"
+#include "kmos_oscan.h"
 
 /*-----------------------------------------------------------------------------
  *                          Functions prototypes
@@ -63,50 +64,6 @@ static char kmos_dark_description[] =
 "It is recommended to provide three or more dark exposures to produce a\n"
 "reasonable master with associated noise.\n"
 "\n"
-"BASIC PARAMETERS\n"
-"----------------\n"
-"--pos_bad_pix_rej\n"
-"--neg_bad_pix_rej\n"
-"Bad pixels above and below defined positive/negative threshold levels will\n"
-"be flagged and output to the BADPIX_DARK frame.\n"
-"The number of bad pixels is returned as a QC1 parameter.\n"
-"The two parameters can be used to change these thresholds.\n"
-"\n"
-"--cmethod\n"
-"Following methods of frame combination are available:\n"
-"   * 'ksigma' (Default)\n"
-"       An iterative sigma clipping. For each position all pixels in the\n"
-"       spectrum are examined. If they deviate significantly, they will be\n"
-"       rejected according to the conditions:\n"
-"           val > mean + stdev * cpos_rej\n"
-"       and\n"
-"           val < mean - stdev * cneg_rej\n"
-"       where --cpos_rej, --cneg_rej and --citer are the wished parameters\n"
-"       In the first iteration median and percentile level are used.\n"
-"   * 'median'\n"
-"       At each pixel position the median is calculated.\n"
-"   * 'average'\n"
-"       At each pixel position the average is calculated.\n"
-"   * 'sum'\n"
-"       At each pixel position the sum is calculated.\n"
-"   * 'min_max'\n"
-"       The specified number of min and max pixel values will be rejected.\n"
-"       --cmax and --cmin apply to this method.\n"
-"\n"
-"--file_extension\n"
-"Set this parameter to TRUE if the EXPTIME keyword should be appended to\n"
-"the output filenames.\n"
-"\n"
-"ADVANCED PARAMETERS\n"
-"-------------------\n"
-"--cpos_rej\n"
-"--cneg_rej\n"
-"--citer\n"
-"   see --cmethod='ksigma'\n"
-"--cmax\n"
-"--cmin\n"
-"   see --cmethod='min_max'\n"
-"\n"
 "---------------------------------------------------------------------------\n"
 "Input files:\n"
 "   DO CATG          Type   Explanation                     Required #Frames\n"
@@ -192,6 +149,13 @@ static int kmos_dark_create(cpl_plugin *plugin)
     recipe->parameters = cpl_parameterlist_new();
 
     /* Fill the parameters list */
+    /* --oscan */
+    p = cpl_parameter_new_value("kmos.kmos_dark.oscan",
+            CPL_TYPE_BOOL, "Apply Overscan Correction",
+            "kmos.kmos_dark", TRUE);
+    cpl_parameter_set_alias(p, CPL_PARAMETER_MODE_CLI, "oscan");
+    cpl_parameter_disable(p, CPL_PARAMETER_MODE_ENV);
+    cpl_parameterlist_append(recipe->parameters, p);
 
     /* --pos_bad_pix_rej */
     p = cpl_parameter_new_value("kmos.kmos_dark.pos_bad_pix_rej", 
@@ -278,7 +242,7 @@ static int kmos_dark(cpl_parameterlist * parlist, cpl_frameset * frameset)
     const char          *   cmethod ;
     const char          *   my_method ;
     cpl_frame           *   frame ;
-    int                     file_extension, citer, cmin, cmax ;
+    int                     file_extension, citer, cmin, cmax, oscan ;
     double                  pos_bad_pix_rej, neg_bad_pix_rej, cneg_rej,
                             cpos_rej, exptime, gain, qc_dark, qc_dark_median, 
                             qc_readnoise, qc_readnoise_median, qc_bad_pix_num ;
@@ -287,6 +251,8 @@ static int kmos_dark(cpl_parameterlist * parlist, cpl_frameset * frameset)
     char                *   extname ;
     char                *   exptimeStr ;
     cpl_imagelist       *   detector_in_window ;
+    cpl_image           *   img_in ;
+    cpl_image           *   img_in_oscan ;
     cpl_image           *   img_in_window ;
     cpl_image           *   combined_data_window ;
     cpl_image           *   combined_data ;
@@ -309,6 +275,8 @@ static int kmos_dark(cpl_parameterlist * parlist, cpl_frameset * frameset)
     exptime = -1.0 ;
 
     /* Get Parameters */
+    par = cpl_parameterlist_find_const(parlist, "kmos.kmos_dark.oscan");
+    oscan = cpl_parameter_get_bool(par);
     par=cpl_parameterlist_find_const(parlist, "kmos.kmos_dark.pos_bad_pix_rej");
     pos_bad_pix_rej = cpl_parameter_get_double(par);
     par=cpl_parameterlist_find_const(parlist, "kmos.kmos_dark.neg_bad_pix_rej");
@@ -379,11 +347,9 @@ static int kmos_dark(cpl_parameterlist * parlist, cpl_frameset * frameset)
         nz = 0;
         while (frame != NULL) {
             /* Load current detector DARK frames into an imagelist */
-            img_in_window = cpl_image_load_window(cpl_frame_get_filename(frame),
-                    CPL_TYPE_FLOAT, 0, i, KMOS_BADPIX_BORDER+1,
-                    KMOS_BADPIX_BORDER+1, nx-KMOS_BADPIX_BORDER,
-                    ny-KMOS_BADPIX_BORDER) ;
-            if (img_in_window == NULL) {
+            img_in = cpl_image_load(cpl_frame_get_filename(frame),
+                    CPL_TYPE_FLOAT, 0, i) ;
+            if (img_in == NULL) {
                 cpl_free(filename) ;
                 cpl_free(filename_bad) ;
                 cpl_imagelist_delete(detector_in_window) ;
@@ -392,6 +358,30 @@ static int kmos_dark(cpl_parameterlist * parlist, cpl_frameset * frameset)
                 cpl_msg_indent_less() ;
                 return -1 ;
             }
+           
+            /* Overscan Correction */
+            if (oscan) {
+                img_in_oscan = kmos_oscan_correct(img_in) ;
+                if (img_in_oscan == NULL) {
+                    cpl_free(filename) ;
+                    cpl_free(filename_bad) ;
+                    cpl_imagelist_delete(detector_in_window) ;
+                    cpl_image_delete(img_in);
+                    cpl_msg_error(__func__, "Cannot correct Overscan") ;
+                    cpl_error_set(__func__, CPL_ERROR_ILLEGAL_INPUT) ;
+                    cpl_msg_indent_less() ;
+                    return -1 ;
+                }
+                cpl_image_delete(img_in) ;
+                img_in = img_in_oscan ;
+            }
+
+            /* Cut the borderѕ */
+            img_in_window = cpl_image_extract(img_in, KMOS_BADPIX_BORDER+1,
+                    KMOS_BADPIX_BORDER+1, nx-KMOS_BADPIX_BORDER,
+                    ny-KMOS_BADPIX_BORDER) ;
+            cpl_image_delete(img_in) ;
+ 
             cpl_imagelist_set(detector_in_window, img_in_window, nz);
             nz++;
                     
