@@ -246,9 +246,11 @@ static int kmos_extract_spec(
     const char          *   centre_txt ;
     cpl_vector          *   centre ;
     int                     cmin, cmax, valid_ifu, citer, save_mask,
-                            devnr1, devnr2, index_data, index_noise ;
+                            devnr1, index_data, index_noise, 
+                            nr_devices ;
     double                  cpos_rej, cneg_rej, radius, r, x_lo, y_lo,
-                            x_hi, y_hi, cen_x, cen_y ;
+                            x_hi, y_hi, cen_x, cen_y, loc_cen_x, loc_cen_y ;
+    cpl_size                auto_cen_x, auto_cen_y ;
     cpl_imagelist       *   data_in ; 
     cpl_imagelist       *   noise_in ; 
     cpl_image           *   mask ;
@@ -280,7 +282,9 @@ static int kmos_extract_spec(
     par = cpl_parameterlist_find_const(parlist, 
             "kmos.kmos_extract_spec.mask_method");
     mask_method = cpl_parameter_get_string(par) ;
-
+    par = cpl_parameterlist_find_const(parlist,
+                "kmos.kmos_extract_spec.save_mask");
+    save_mask = cpl_parameter_get_bool(par);
     if (!strcmp(mask_method, "integrated")) {
         par = cpl_parameterlist_find_const(parlist, 
                 "kmos.kmos_extract_spec.centre");
@@ -294,11 +298,6 @@ static int kmos_extract_spec(
         cen_x = cpl_vector_get(centre, 0);
         cen_y = cpl_vector_get(centre, 1);
         cpl_vector_delete(centre);
-        if (cen_x < 0.0 || cen_y < 0.0) {
-            cpl_msg_error(__func__, "centre must be greater than 0.0") ;
-            cpl_error_set(__func__, CPL_ERROR_ILLEGAL_INPUT) ;
-            return -1 ;
-        }
         par = cpl_parameterlist_find_const(parlist,
                 "kmos.kmos_extract_spec.radius");
         radius = cpl_parameter_get_double(par) ;
@@ -310,9 +309,6 @@ static int kmos_extract_spec(
     } else if (strcmp(mask_method, "optimal") == 0) {
         kmos_combine_pars_load(parlist, "kmos.kmos_extract_spec", &cmethod,
                 &cpos_rej, &cneg_rej, &citer, &cmin, &cmax, FALSE);
-        par = cpl_parameterlist_find_const(parlist,
-                    "kmos.kmos_extract_spec.save_mask");
-        save_mask = cpl_parameter_get_bool(par);
     } else {
         cpl_msg_error(__func__, "Unsupported mask method: %s", mask_method) ;
         cpl_error_set(__func__, CPL_ERROR_ILLEGAL_INPUT) ;
@@ -345,49 +341,29 @@ static int kmos_extract_spec(
     } else {
         op2_frame = NULL ;
     }
-
-    /* --- load, update & save primary header --- */
+    
+   /* --- load, update & save primary header --- */
     kmo_dfs_save_main_header(frameset, EXTRACT_SPEC, "", op1_frame, NULL, 
             parlist, cpl_func);
     if (save_mask) {
         kmo_dfs_save_main_header(frameset, EXTRACT_SPEC_MASK, "", op1_frame, 
                 NULL, parlist, cpl_func);
     }
-
-    /* Create mask for integrated-method */
-    if (!strcmp(mask_method, "integrated")) {
-        mask = cpl_image_new(desc1.naxis1, desc1.naxis2, CPL_TYPE_FLOAT);
-        kmo_image_fill(mask,0.0);
-        pmask = cpl_image_get_data_float(mask);
-
-        /* draw circle */
-        x_lo = floor(cen_x - radius);
-        if (x_lo < 0) x_lo = 0;
-        y_lo = floor(cen_y - radius);
-        if (y_lo < 0) y_lo = 0;
-        x_hi = ceil(cen_x + radius);
-        if (x_hi > desc1.naxis1) x_hi = desc1.naxis1;
-        y_hi = ceil(cen_y + radius);
-        if (y_hi > desc1.naxis2) y_hi = desc1.naxis2;
-        for (x = x_lo; x < x_hi; x++) {
-            for (y = y_lo; y < y_hi; y++) {
-                r = sqrt(pow(x - cen_x,2) + pow(y - cen_y,2));
-                if (r <= radius) pmask[x + y * desc1.naxis1] = 1.0;
-            }
-        }            
+     
+    /* Number of extensions to extract */
+    if (desc1.ex_noise == TRUE) {
+        nr_devices = desc1.nr_ext / 2;
+    } else {
+        nr_devices = desc1.nr_ext;
     }
-    
-    /* Loop on detectors */
-    for (i = 1; i <= KMOS_NR_IFUS ; i++) {
+
+    /* Loop on the devices */
+    for (i = 1; i <= nr_devices ; i++) {
         if (desc1.ex_noise == FALSE) {
             devnr1 = desc1.sub_desc[i - 1].device_nr;
         } else {
             devnr1 = desc1.sub_desc[2 * i - 1].device_nr;
         }
-        /* Mask doesn't contain any noise extensions */
-        if (strcmp(mask_method, "mask") == 0) {
-            devnr2 = desc2.sub_desc[i - 1].device_nr;
-        }
 
         if (desc1.ex_badpix == FALSE) {
             index_data = kmo_identify_index_desc(desc1, devnr1, FALSE);
@@ -422,11 +398,10 @@ static int kmos_extract_spec(
                 noise_in = NULL ;
             }
 
-            // create or load mask (for integrated-method already
-            // done outside the for-loop)
+            /* Create the mask */
             if (!strcmp(mask_method, "mask")) {
-                mask = kmo_dfs_load_image(frameset, "1", devnr2, FALSE, 
-                        FALSE, NULL);
+                mask = kmo_dfs_load_image(frameset, "1", 
+                        desc2.sub_desc[i - 1].device_nr, FALSE, FALSE, NULL);
             } else if (!strcmp(mask_method, "optimal")) {
                 kmclipm_make_image(data_in, NULL, &made_data_img, NULL, NULL,
                         cmethod, cpos_rej, cneg_rej, citer, cmax, cmin);
@@ -442,6 +417,37 @@ static int kmos_extract_spec(
                 cpl_image_divide_scalar(mask, cpl_vector_get(fit_par, 1));
                 cpl_vector_delete(fit_par); 
                 cpl_image_delete(made_data_img);
+            } else if (!strcmp(mask_method, "integrated")) {
+                if (cen_x < 1.0 || cen_y < 1.0) {
+                    kmclipm_make_image(data_in, NULL, &made_data_img, NULL, 
+                            NULL, "median", 3.0, 3.0, 3, 1, 1);
+                    cpl_image_get_maxpos(made_data_img,&auto_cen_x,&auto_cen_y);
+                    loc_cen_x = (double)auto_cen_x - 1.0 ;
+                    loc_cen_y = (double)auto_cen_y - 1.0 ;
+                    cpl_image_delete(made_data_img);
+                } else {
+                    loc_cen_x = cen_x - 1.0 ;
+                    loc_cen_y = cen_y - 1.0 ;
+                }
+                mask = cpl_image_new(desc1.naxis1, desc1.naxis2,CPL_TYPE_FLOAT);
+                kmo_image_fill(mask,0.0);
+                pmask = cpl_image_get_data_float(mask);
+
+                /* draw circle */
+                x_lo = floor(loc_cen_x - radius);
+                if (x_lo < 0) x_lo = 0;
+                y_lo = floor(loc_cen_y - radius);
+                if (y_lo < 0) y_lo = 0;
+                x_hi = ceil(loc_cen_x + radius);
+                if (x_hi > desc1.naxis1) x_hi = desc1.naxis1;
+                y_hi = ceil(loc_cen_y + radius);
+                if (y_hi > desc1.naxis2) y_hi = desc1.naxis2;
+                for (x = x_lo; x < x_hi; x++) {
+                    for (y = y_lo; y < y_hi; y++) {
+                        r = sqrt(pow(x - loc_cen_x,2) + pow(y - loc_cen_y,2));
+                        if (r <= radius) pmask[x + y * desc1.naxis1] = 1.0;
+                    }
+                }            
             }
 
             /* Process & save data */
@@ -486,7 +492,7 @@ static int kmos_extract_spec(
             }
             cpl_imagelist_delete(data_in); 
             cpl_imagelist_delete(noise_in);
-            if (strcmp(mask_method, "integrated")) cpl_image_delete(mask);
+            cpl_image_delete(mask);
         } else {
             /* Invalid IFU */
             kmo_dfs_save_sub_header(EXTRACT_SPEC, "", sub_header_data);
@@ -497,7 +503,6 @@ static int kmos_extract_spec(
         cpl_propertylist_delete(sub_header_data); 
         if (desc1.ex_noise) cpl_propertylist_delete(sub_header_noise);
     }
-    if (!strcmp(mask_method, "integrated")) cpl_image_delete(mask);
     kmo_free_fits_desc(&desc1);
     kmo_free_fits_desc(&desc2);
     return 0 ;
