@@ -376,8 +376,7 @@ static int kmos_flat(cpl_parameterlist *parlist, cpl_frameset *frameset)
     int                     cmax, cmin, citer ;
     const char          *   cmethod ;
     cpl_frame           *   frame ;
-    cpl_propertylist    *   plist ;
-    int                     nx, ny, ne ;
+    int                     nx, ny, next ;
     int                 *   angles_array ;
     int                     nb_angles ;
     cpl_image           **  stored_flat ;
@@ -472,7 +471,7 @@ static int kmos_flat(cpl_parameterlist *parlist, cpl_frameset *frameset)
     }
 
     /* Check the inputs consistency */
-    if (kmos_flat_check_inputs(frameset, &nx, &ny, &ne, &exptime) != 1) {
+    if (kmos_flat_check_inputs(frameset, &nx, &ny, &next, &exptime) != 1) {
         cpl_msg_error(__func__, "Input frameset is not consistent") ;
         cpl_error_set(__func__, CPL_ERROR_ILLEGAL_INPUT) ;
         return -1 ;
@@ -492,25 +491,25 @@ static int kmos_flat(cpl_parameterlist *parlist, cpl_frameset *frameset)
     /* The frames have to be stored temporarily because the QC parameters */
     /* for the main header are calculated from each detector.  */
     /* So they can be stored only when all detectors are processed */
-    stored_flat = (cpl_image**)cpl_calloc(ne*nb_angles, sizeof(cpl_image*));
-    stored_noise = (cpl_image**)cpl_calloc(ne * nb_angles, sizeof(cpl_image*));
-    stored_badpix = (cpl_image**)cpl_calloc(ne * nb_angles, sizeof(cpl_image*));
-    stored_xcal = (cpl_image**)cpl_calloc(ne * nb_angles, sizeof(cpl_image*));
-    stored_ycal = (cpl_image**)cpl_calloc(ne * nb_angles, sizeof(cpl_image*));
-    stored_qc_flat_sat = (int*)cpl_malloc(ne * nb_angles * sizeof(int));
-    stored_qc_flat_eff = (double*)cpl_malloc(ne * nb_angles * sizeof(double));
-    stored_qc_flat_sn = (double*)cpl_malloc(ne * nb_angles * sizeof(double));
-    stored_gapmean = (double*)cpl_malloc(ne * nb_angles * sizeof(double));
-    stored_gapsdv = (double*)cpl_malloc(ne * nb_angles * sizeof(double));
-    stored_gapmaxdev = (double*)cpl_malloc(ne * nb_angles * sizeof(double));
-    stored_slitmean = (double*)cpl_malloc(ne * nb_angles * sizeof(double));
-    stored_slitsdv = (double*)cpl_malloc(ne * nb_angles * sizeof(double));
-    stored_slitmaxdev = (double*)cpl_malloc(ne * nb_angles * sizeof(double));
-    spec_found = (cpl_error_code*)cpl_malloc(ne * nb_angles * 
+    stored_flat = (cpl_image**)cpl_calloc(next*nb_angles, sizeof(cpl_image*));
+    stored_noise = (cpl_image**)cpl_calloc(next*nb_angles, sizeof(cpl_image*));
+    stored_badpix = (cpl_image**)cpl_calloc(next*nb_angles, sizeof(cpl_image*));
+    stored_xcal = (cpl_image**)cpl_calloc(next*nb_angles, sizeof(cpl_image*));
+    stored_ycal = (cpl_image**)cpl_calloc(next * nb_angles, sizeof(cpl_image*));
+    stored_qc_flat_sat = (int*)cpl_malloc(next * nb_angles * sizeof(int));
+    stored_qc_flat_eff = (double*)cpl_malloc(next * nb_angles * sizeof(double));
+    stored_qc_flat_sn = (double*)cpl_malloc(next * nb_angles * sizeof(double));
+    stored_gapmean = (double*)cpl_malloc(next * nb_angles * sizeof(double));
+    stored_gapsdv = (double*)cpl_malloc(next * nb_angles * sizeof(double));
+    stored_gapmaxdev = (double*)cpl_malloc(next * nb_angles * sizeof(double));
+    stored_slitmean = (double*)cpl_malloc(next * nb_angles * sizeof(double));
+    stored_slitsdv = (double*)cpl_malloc(next * nb_angles * sizeof(double));
+    stored_slitmaxdev = (double*)cpl_malloc(next * nb_angles * sizeof(double));
+    spec_found = (cpl_error_code*)cpl_malloc(next * nb_angles * 
             sizeof(cpl_error_code));
 
     /* Initialise */
-    for (i = 0; i < ne * nb_angles ; i++) {
+    for (i = 0; i < next * nb_angles ; i++) {
         stored_qc_flat_sat[i] = 0;
         stored_qc_flat_eff[i] = 0.0;
         stored_qc_flat_sn[i] = 0.0;
@@ -524,8 +523,8 @@ static int kmos_flat(cpl_parameterlist *parlist, cpl_frameset *frameset)
     }
 
     /* TODO : Improve handling of edge_table !!!! */
-    edge_table = (cpl_table***)cpl_malloc(ne * nb_angles * sizeof(cpl_table**));
-    for (i = 0; i < ne * nb_angles; i++) edge_table[i] = NULL;
+    edge_table = (cpl_table***)cpl_malloc(next*nb_angles * sizeof(cpl_table**));
+    for (i = 0; i < next * nb_angles; i++) edge_table[i] = NULL;
 
     /* Check which IFUs are active for all FLAT_ON frames */
     unused_ifus_before = kmo_get_unused_ifus(frameset, 0, 0);
@@ -534,7 +533,7 @@ static int kmos_flat(cpl_parameterlist *parlist, cpl_frameset *frameset)
     kmo_free_unused_ifus(unused_ifus_before);
 
     /* Combine the FLAT_OFF frames for the 3 detectors */
-    for (i = 1; i <= ne; i++) {
+    for (i = 1; i <= next; i++) {
         /* Compute only one detector */
         if (reduce_det != 0 && i != reduce_det) continue ;
 
@@ -587,14 +586,14 @@ static int kmos_flat(cpl_parameterlist *parlist, cpl_frameset *frameset)
             return -1 ;
         }
 
-        for (i = 1; i <= ne; i++) {
+        for (i = 1; i <= next; i++) {
             /* Compute only one detector */
             if (reduce_det != 0 && i != reduce_det) continue ;
 
             cpl_msg_info(__func__, "Processing detector No. %d", i);
             cpl_msg_indent_more() ;
 
-            sx = a * ne + (i - 1);
+            sx = a * next + (i - 1);
 
             /* Load lamp-on images for Angle a */
             det_lamp_on = cpl_imagelist_new();
@@ -666,10 +665,7 @@ static int kmos_flat(cpl_parameterlist *parlist, cpl_frameset *frameset)
                 mean_data = cpl_image_get_mean(combined_data_on);
                 stored_qc_flat_eff[sx] = mean_data / exptime;
                 mean_noise = cpl_image_get_mean(combined_noise_on);
-
                 if (fabs(mean_noise) < 1e-3) {
-                    cpl_image_save(combined_noise_on, "noise.fits",
-                            CPL_TYPE_FLOAT, NULL, CPL_IO_CREATE);
                     cpl_msg_error(__func__, "Division by 0.0") ;
                     cpl_error_set(__func__, CPL_ERROR_ILLEGAL_INPUT) ;
                     cpl_free(angles_array) ;
@@ -734,13 +730,13 @@ static int kmos_flat(cpl_parameterlist *parlist, cpl_frameset *frameset)
             cpl_image_delete(bad_pix_mask_flat); 
 
             cpl_msg_indent_less() ;
-        } // for i = 1; i <= ne
+        } // for i = 1; i <= next
         cpl_frameset_delete(angle_frameset); 
         cpl_msg_indent_less() ;
     } // for a = 0; a < nb_angles
 
     /* Clean OFF frames */
-    for (i = 1; i <= ne; i++) {
+    for (i = 1; i <= next; i++) {
         /* Compute only one detector */
         if (reduce_det != 0 && i != reduce_det) continue ;
 
@@ -759,7 +755,7 @@ static int kmos_flat(cpl_parameterlist *parlist, cpl_frameset *frameset)
     kmo_free_unused_ifus(unused_ifus_after);
 
     /* xcal gets additionally the boundaries of the IFUs for reconstruction */
-    main_header_xcal=kmos_create_bounds_properties(stored_ycal, ne, nb_angles) ;
+    main_header_xcal=kmos_create_bounds_properties(stored_ycal,next,nb_angles) ;
 
     /* --------- saving headers  */
     if (!suppress_extension)    fn_suffix = cpl_sprintf("%s", suffix);
@@ -789,11 +785,11 @@ static int kmos_flat(cpl_parameterlist *parlist, cpl_frameset *frameset)
         /* Reduce only one angle */
         if (reduce_angle <= 360 && angles_array[a] != reduce_angle) continue ;
         
-        for (i = 1; i <= ne; i++) {
+        for (i = 1; i <= next; i++) {
             /* Compute only one detector */
             if (reduce_det != 0 && i != reduce_det) continue ;
             
-            sx = a * ne + (i - 1);
+            sx = a * next + (i - 1);
             // load stored data again
             stored_flat[sx]=kmclipm_image_load(fn_flat, CPL_TYPE_FLOAT, 0, 
                     ext_index);
@@ -937,7 +933,7 @@ static int kmos_flat(cpl_parameterlist *parlist, cpl_frameset *frameset)
             cpl_image_delete(stored_flat[sx]); 
             cpl_image_delete(stored_noise[sx]);
             cpl_image_delete(stored_badpix[sx]);
-        } // for (i = ne)
+        } // for (i = next)
     } // for (a = nb_angles)
     
     // delete temporary files
@@ -958,14 +954,14 @@ static int kmos_flat(cpl_parameterlist *parlist, cpl_frameset *frameset)
     cpl_free(stored_flat);
     cpl_free(stored_noise);
     cpl_free(stored_badpix);
-    for (i = 0; i < ne * nb_angles; i++) {
+    for (i = 0; i < next * nb_angles; i++) {
         cpl_image_delete(stored_xcal[i]);
         cpl_image_delete(stored_ycal[i]);
     }
     cpl_free(stored_xcal);
     cpl_free(stored_ycal);
     if (edge_table != NULL) {
-        for (i = 0; i < ne * nb_angles; i++) {
+        for (i = 0; i < next * nb_angles; i++) {
             if (edge_table[i] != NULL) {
                 for (j = 0; j < KMOS_IFUS_PER_DETECTOR; j++) {
                     cpl_table_delete(edge_table[i][j]);
@@ -989,14 +985,14 @@ static int kmos_flat(cpl_parameterlist *parlist, cpl_frameset *frameset)
   @param    frameset        Set of frames
   @param    nx [out]        images x size
   @param    ny [out]        images y size
-  @param    ne [out]        Nb extensions
+  @param    next [out]        Nb extensions
   @param    exptime_on [out] exptime
   @return   1 if consistent, 0 if not, -1 in error case
  */
 /*----------------------------------------------------------------------------*/
 static cpl_propertylist * kmos_create_bounds_properties(
         cpl_image           **  stored_ycal,
-        int                     ne,
+        int                     next,
         int                     nb_angles)
 {
     cpl_propertylist    *   bounds_props ;
@@ -1012,8 +1008,8 @@ static cpl_propertylist * kmos_create_bounds_properties(
     bounds_props = cpl_propertylist_new();
 
     /* Initialize total_bounds */
-    total_bounds = (int**)cpl_malloc(ne*sizeof(int*));
-    for (i = 0; i < ne; i++) {
+    total_bounds = (int**)cpl_malloc(next*sizeof(int*));
+    for (i = 0; i < next; i++) {
         total_bounds[i]=(int*)cpl_calloc(2*KMOS_IFUS_PER_DETECTOR,sizeof(int));
         for (j = 0; j < KMOS_IFUS_PER_DETECTOR; j++) {
             total_bounds[i][2*j] = 2048;
@@ -1023,8 +1019,8 @@ static cpl_propertylist * kmos_create_bounds_properties(
 
     /* Store the min left bound and max right bound for all angles */
     for (a = 0; a < nb_angles; a++) {
-        for (i = 0; i < ne; i++) {
-            sx = a * ne + i;
+        for (i = 0; i < next; i++) {
+            sx = a * next + i;
             if (stored_ycal[sx] != NULL) {
                 bounds = kmo_split_frame(stored_ycal[sx]);
 
@@ -1054,12 +1050,12 @@ static cpl_propertylist * kmos_create_bounds_properties(
                     total_bounds[i][2*j+1] = -1;
                 }
             }
-        } // for (ne)
+        } // for (next)
     } // for (nb_angles)
 
     /* Write the min left bound and max right bound for all angles */
     /* into the main header */
-    for (i = 0; i < ne; i++) {
+    for (i = 0; i < next; i++) {
         for (j = 0; j < KMOS_IFUS_PER_DETECTOR; j++) {
             if (total_bounds[i][2*j] > -1) {
                 tmpstr= cpl_sprintf("%s%d%s", BOUNDS_PREFIX, 
@@ -1079,8 +1075,8 @@ static cpl_propertylist * kmos_create_bounds_properties(
                 cpl_free(tmpstr);
             }
         }
-    } // for (ne)
-    for (i = 0; i < ne; i++) cpl_free(total_bounds[i]);
+    } // for (next)
+    for (i = 0; i < next; i++) cpl_free(total_bounds[i]);
     cpl_free(total_bounds);
 
     return bounds_props ;
@@ -1092,7 +1088,7 @@ static cpl_propertylist * kmos_create_bounds_properties(
   @param    frameset        Set of frames
   @param    nx [out]        images x size
   @param    ny [out]        images y size
-  @param    ne [out]        Nb extensions
+  @param    next [out]        Nb extensions
   @param    exptime_on [out] exptime
   @return   1 if consistent, 0 if not, -1 in error case
  */
@@ -1101,7 +1097,7 @@ static int kmos_flat_check_inputs(
         cpl_frameset        *   frameset,
         int                 *   nx,
         int                 *   ny,
-        int                 *   ne,
+        int                 *   next,
         double              *   exptime_on)
 {
     cpl_frame           *   frame ;
@@ -1111,7 +1107,7 @@ static int kmos_flat_check_inputs(
     int                     ndit ;
     double                  exptime ;
     const char          *   readmode ;
-    int                     naxis1, naxis2, next ;
+    int                     naxis1, naxis2, n_ext ;
 
     /* TODO Add frames dimensions checks TODO */
 
@@ -1125,8 +1121,8 @@ static int kmos_flat_check_inputs(
         cpl_msg_warning(__func__, "BADPIXEL_DARK frame is missing") ;
         return 0 ;
     }
-    next = cpl_frame_get_nextensions(frame);
-    if (next != KMOS_NR_DETECTORS) {
+    n_ext = cpl_frame_get_nextensions(frame);
+    if (n_ext != KMOS_NR_DETECTORS) {
         cpl_msg_warning(__func__, "BADPIXEL_DARK must have 3 extensions") ;
         return 0 ;
     }
@@ -1243,7 +1239,7 @@ static int kmos_flat_check_inputs(
     /* Return */
     *nx = naxis1 ;
     *ny = naxis2 ;
-    *ne = next ;
+    *next = n_ext ;
     *exptime_on = exptime ;
     return 1 ;
 }
