@@ -41,6 +41,7 @@
 #include "kmo_utils.h"
 #include "kmo_constants.h"
 #include "kmo_debug.h"
+#include "kmos_oscan.h"
 
 /*-----------------------------------------------------------------------------
  *                          Functions prototypes
@@ -236,6 +237,14 @@ static int kmos_reconstruct_create(cpl_plugin *plugin)
     cpl_parameter_disable(p, CPL_PARAMETER_MODE_ENV);
     cpl_parameterlist_append(recipe->parameters, p);
 
+    /* --oscan */
+    p = cpl_parameter_new_value("kmos.kmos_reconstruct.oscan",
+            CPL_TYPE_BOOL, "Apply Overscan Correction",
+            "kmos.kmos_reconstruct", TRUE);
+    cpl_parameter_set_alias(p, CPL_PARAMETER_MODE_CLI, "oscan");
+    cpl_parameter_disable(p, CPL_PARAMETER_MODE_ENV);
+    cpl_parameterlist_append(recipe->parameters, p);
+
     /* Add parameters for band-definition */
     kmos_band_pars_create(recipe->parameters, "kmos.kmos_reconstruct");
     return 0;
@@ -299,7 +308,7 @@ static int kmos_reconstruct(cpl_parameterlist *parlist, cpl_frameset *frameset)
     const cpl_parameter *   par ;
     const char          *   imethod ;
     int                     flux, detectorimage, xcal_interpolation,
-                            file_extension ;
+                            file_extension, oscan ;
     double                  neighborhoodRange, pix_scale, scaling ;
     cpl_frame           *   input_frame ;
     cpl_frame           *   xcal_frame ;
@@ -362,6 +371,8 @@ static int kmos_reconstruct(cpl_parameterlist *parlist, cpl_frameset *frameset)
     par = cpl_parameterlist_find_const(parlist,
             "kmos.kmos_reconstruct.xcal_interpolation");
     xcal_interpolation = cpl_parameter_get_bool(par);
+    par = cpl_parameterlist_find_const(parlist, "kmos.kmos_reconstruct.oscan");
+    oscan = cpl_parameter_get_bool(par);
 
     /* Check Parameters */
     if (strcmp(imethod, "NN") && strcmp(imethod, "lwNN") &&
@@ -530,7 +541,7 @@ static int kmos_reconstruct(cpl_parameterlist *parlist, cpl_frameset *frameset)
                         bounds[2*(ifu_nr-1)+1], input_frame, input_frame_name,
                         NULL, NULL, NULL, xcal_frame, ycal_frame, lcal_frame,
                         NULL, NULL, &gd, &cube_data, &cube_noise, flux,
-                        0, xcal_interpolation);
+                        0, xcal_interpolation, oscan);
 
                 /* Reconstruct again using OH_SPEC */
                 if (ref_spectrum_frame != NULL && cube_data != NULL) {
@@ -543,7 +554,7 @@ static int kmos_reconstruct(cpl_parameterlist *parlist, cpl_frameset *frameset)
                             input_frame_name, NULL, NULL, NULL, xcal_frame, 
                             ycal_frame, lcal_frame, lcorr_coeffs, NULL, &gd, 
                             &cube_data, &cube_noise, flux, 0, 
-                            xcal_interpolation);
+                            xcal_interpolation, oscan);
                     cpl_polynomial_delete(lcorr_coeffs);
                 }
 
