@@ -180,12 +180,16 @@ static int kmos_std_star_compute_ifu(
         const char          *   fmethod,
         int                     flux,
         int                     xcal_interpolation,
+        const char          *   mask_method,
         const char          *   cmethod,
         double                  cpos_rej,
         double                  cneg_rej,
         int                     citer,
         int                     cmax,
         int                     cmin,
+        double                  cen_x,
+        double                  cen_y,
+        double                  radius,
         const char          *   filter_id,
         char                    star_type,
         int                     no_noise,
@@ -442,6 +446,30 @@ static int kmos_std_star_create(cpl_plugin *plugin)
     /* Add parameters for band-definition */
     kmos_band_pars_create(recipe->parameters, "kmos.kmos_std_star");
 
+    /* --mask_method */
+    p = cpl_parameter_new_value("kmos.kmos_std_star.mask_method",
+            CPL_TYPE_STRING, "Method used : mask, integrated or optimal",
+            "kmos.kmos_std_star", "optimal");
+    cpl_parameter_set_alias(p, CPL_PARAMETER_MODE_CLI, "mask_method");
+    cpl_parameter_disable(p, CPL_PARAMETER_MODE_ENV);
+    cpl_parameterlist_append(recipe->parameters, p);
+
+    /* --centre */
+    p = cpl_parameter_new_value("kmos.kmos_std_star.centre",
+            CPL_TYPE_STRING, "The centre of the circular mask (pixel)",
+            "kmos.kmos_std_star", "7.5,7.5");
+    cpl_parameter_set_alias(p, CPL_PARAMETER_MODE_CLI, "centre");
+    cpl_parameter_disable(p, CPL_PARAMETER_MODE_ENV);
+    cpl_parameterlist_append(recipe->parameters, p);
+
+    /* --radius */
+    p = cpl_parameter_new_value("kmos.kmos_std_star.radius",
+            CPL_TYPE_DOUBLE, "The radius of the circular mask (pixel)",
+            "kmos.kmos_std_star", 3.0);
+    cpl_parameter_set_alias(p, CPL_PARAMETER_MODE_CLI, "radius");
+    cpl_parameter_disable(p, CPL_PARAMETER_MODE_ENV);
+    cpl_parameterlist_append(recipe->parameters, p);
+
     /* Add parameters for combining */
     return kmos_combine_pars_create(recipe->parameters, "kmos.kmos_std_star",
             DEF_REJ_METHOD, FALSE);
@@ -508,9 +536,13 @@ static int kmos_std_star(cpl_parameterlist *parlist, cpl_frameset *frameset)
     const char          *   imethod ;        
     const char          *   cmethod ;        
     const char          *   fmethod ;        
+    const char          *   mask_method ;        
+    const char          *   centre_txt ;
+    cpl_vector          *   centre ;
     const char          *   spec_type ;        
     const char          *   magnitude_txt ;        
-    double                  cpos_rej, cneg_rej, neighborhoodRange ;
+    double                  cpos_rej, cneg_rej, neighborhoodRange, cen_x, cen_y,
+                            radius ;
     int                     flux, save_cubes, no_noise, citer, cmin, cmax,
                             xcal_interpolation, suppress_extension ;
     /*********************/
@@ -611,8 +643,35 @@ static int kmos_std_star(cpl_parameterlist *parlist, cpl_frameset *frameset)
         "kmos.kmos_std_star.suppress_extension");
     suppress_extension = cpl_parameter_get_bool(par);
     kmos_band_pars_load(parlist, "kmos.kmos_std_star");
-    kmos_combine_pars_load(parlist, "kmos.kmos_std_star", &cmethod, &cpos_rej,
-            &cneg_rej, &citer, &cmin, &cmax, FALSE);
+    par = cpl_parameterlist_find_const(parlist,
+            "kmos.kmos_std_star.mask_method");
+    mask_method = cpl_parameter_get_string(par) ;
+    if (!strcmp(mask_method, "integrated")) {
+        par = cpl_parameterlist_find_const(parlist,"kmos.kmos_std_star.centre");
+        centre_txt = cpl_parameter_get_string(par) ;
+        centre = kmo_identify_ranges(centre_txt);
+        if (cpl_vector_get_size(centre) != 2) {
+            cpl_msg_error(__func__, "centre must have 2 values like a,b") ;
+            cpl_error_set(__func__, CPL_ERROR_ILLEGAL_INPUT) ;
+            return -1 ;
+        }
+        cen_x = cpl_vector_get(centre, 0);
+        cen_y = cpl_vector_get(centre, 1);
+        cpl_vector_delete(centre);
+        par = cpl_parameterlist_find_const(parlist,"kmos.kmos_std_star.radius");
+        radius = cpl_parameter_get_double(par) ;
+        if (radius < 0.0) {
+            cpl_msg_error(__func__, "radius must be greater than 0.0") ;
+            cpl_error_set(__func__, CPL_ERROR_ILLEGAL_INPUT) ;
+            return -1 ;
+        }
+    } else if (strcmp(mask_method, "optimal")) {
+        cpl_msg_error(__func__, "Unsupported mask method: %s", mask_method) ;
+        cpl_error_set(__func__, CPL_ERROR_ILLEGAL_INPUT) ;
+        return -1 ;
+    }
+    kmos_combine_pars_load(parlist, "kmos.kmos_std_star", &cmethod,
+            &cpos_rej, &cneg_rej, &citer, &cmin, &cmax, FALSE);
 
     /* Check Parameters */
     if (strcmp(cmethod, "average") && strcmp(cmethod, "median") &&
@@ -757,7 +816,8 @@ static int kmos_std_star(cpl_parameterlist *parlist, cpl_frameset *frameset)
     kmo_free_unused_ifus(unused_ifus_after);
     cpl_frameset_delete(frameset_std) ;
     return 0 ;
-*/
+    */
+
     /* Get the first frame containing object */
     obj_frame = obj_sky_struct->table[0].objFrame;
     main_header_tel = kmclipm_propertylist_load(
@@ -820,10 +880,10 @@ static int kmos_std_star(cpl_parameterlist *parlist, cpl_frameset *frameset)
                             lcal_frame, illum_frame, atmos_frame, solar_frame, 
                             ifu_nr, main_header_tel, gd, bounds[2*(ifu_nr-1)], 
                             bounds[2*(ifu_nr-1)+1], fmethod, flux, 
-                            xcal_interpolation, cmethod, cpos_rej, cneg_rej, 
-                            citer, cmax, cmin, filter_id, star_type, no_noise, 
-                            is_stdstarscipatt, sky_sky_struct[ifu_nr-1], 
-                            star_temp, 
+                            xcal_interpolation, mask_method, cmethod, cpos_rej,
+                            cneg_rej, citer, cmax, cmin, cen_x, cen_y, radius, 
+                            filter_id, star_type, no_noise, is_stdstarscipatt, 
+                            sky_sky_struct[ifu_nr-1], star_temp, 
                             &spec_qc,
                             &stored_sub_tel_data_headers[ifu_nr-1],
                             &stored_sub_psf_headers[ifu_nr-1],
@@ -973,12 +1033,12 @@ static int kmos_std_star(cpl_parameterlist *parlist, cpl_frameset *frameset)
         /* QC THROUGHPUT MEAN and QC THROUGHPUT SDV */
         kmo_calc_mean_throughput(stored_qc_throughput, nifus, &throughput_mean,
                 &throughput_sdv);
-        cpl_free(stored_qc_throughput);
         kmclipm_update_property_double(main_header_tel, QC_THROUGHPUT_MEAN,
                 throughput_mean, "[] mean throughput for all detectors");
         kmclipm_update_property_double(main_header_tel, QC_THROUGHPUT_SDV,
                 throughput_sdv, "[] stdev throughput for all detectors");
     }
+    cpl_free(stored_qc_throughput);
     
     /* Save output data */
     if (!suppress_extension)    fn_suffix = cpl_sprintf("%s", suffix);
@@ -1148,7 +1208,6 @@ static int kmos_std_star(cpl_parameterlist *parlist, cpl_frameset *frameset)
 
 /**@}*/
 
-
 /*----------------------------------------------------------------------------*/
 /**
   @brief    Process IFU
@@ -1174,12 +1233,16 @@ static int kmos_std_star_compute_ifu(
         const char          *   fmethod,
         int                     flux,
         int                     xcal_interpolation,
+        const char          *   mask_method,
         const char          *   cmethod,
         double                  cpos_rej,
         double                  cneg_rej,
         int                     citer,
         int                     cmax,
         int                     cmin,
+        double                  cen_x,
+        double                  cen_y,
+        double                  radius,
         const char          *   filter_id,
         char                    star_type,
         int                     no_noise,
@@ -1204,7 +1267,9 @@ static int kmos_std_star_compute_ifu(
     cpl_propertylist    *   tmp_head ;
     cpl_image           *   illum_corr ;
     cpl_propertylist    *   pl_psf ;
-    double                  std_trace, factor_fwhm, spat_res ;
+    double                  std_trace, factor_fwhm, spat_res, x_lo, y_lo, x_hi,
+                            y_hi, loc_cen_x, loc_cen_y, r ;
+    cpl_size                auto_cen_x, auto_cen_y, nx, ny ;
     cpl_vector          *   tmp_vec ;
     cpl_vector          *   tmp_spec_data ;
     cpl_vector          *   tmp_spec_noise ;
@@ -1218,10 +1283,11 @@ static int kmos_std_star_compute_ifu(
     cpl_vector          **  vec_array ;
     const double        *   ptmp_spec_data ;
     double              *   ptmp_spec_noise ;
+    float               *   pmask ;
     cpl_imagelist       *   tmp_cube ;
     cpl_vector          *   lambda_x ;
     cpl_vector          *   atmos_model ;
-    int                     i, j, k, nr_sky_pairs, npix ;
+    int                     i, j, k, nr_sky_pairs, npix, x, y ;
  
     /* Check inputs */
     
@@ -1290,7 +1356,7 @@ static int kmos_std_star_compute_ifu(
     kmo_reconstruct_sci(ifu_nr, low_bound, high_bound, obj_frame, STD,
             sky_frame, STD, flat_frame, xcal_frame, ycal_frame, lcal_frame,
             NULL, NULL, &gd, out_data_cube, out_noise_cube, flux, FALSE,
-            xcal_interpolation);
+            xcal_interpolation, FALSE);
 
     /* Illumination correction */
     /* Illumination noise small versus noise - skipped */
@@ -1313,14 +1379,45 @@ static int kmos_std_star_compute_ifu(
     /* Fit a 2D profile to get a mask and fwhm in x and y */
     tmp_vec = kmo_fit_profile_2D(*out_psf_data, NULL, fmethod,out_mask,&pl_psf);
 
-    /* Normalise mask to 1 and clip values below 0.5 */
-    cpl_image_divide_scalar(*out_mask, cpl_image_get_max(*out_mask));
-    for (i = 1; i <= cpl_image_get_size_x(*out_mask); i++) {
-        for (j = 1; j <= cpl_image_get_size_y(*out_mask); j++) {
-            if (cpl_image_get(*out_mask, i, j, &k) < 0.5) 
-                cpl_image_set(*out_mask, i, j, 0.);
-            else   
-                cpl_image_set(*out_mask, i, j, 1.);
+    if (!strcmp(mask_method, "integrated")) {
+        if (cen_x < 1.0 || cen_y < 1.0) {
+            cpl_image_get_maxpos(*out_psf_data,&auto_cen_x,&auto_cen_y);
+            loc_cen_x = (double)auto_cen_x - 1.0 ;
+            loc_cen_y = (double)auto_cen_y - 1.0 ;
+        } else {
+            loc_cen_x = cen_x - 1.0 ;
+            loc_cen_y = cen_y - 1.0 ;
+        }
+        kmo_image_fill(*out_mask,0.0);
+        pmask = cpl_image_get_data_float(*out_mask);
+        nx = cpl_image_get_size_x(*out_mask);
+        ny = cpl_image_get_size_y(*out_mask);
+
+        /* draw circle */
+        x_lo = floor(loc_cen_x - radius);
+        if (x_lo < 0) x_lo = 0;
+        y_lo = floor(loc_cen_y - radius);
+        if (y_lo < 0) y_lo = 0;
+        x_hi = ceil(loc_cen_x + radius);
+        if (x_hi > nx) x_hi = nx;
+        y_hi = ceil(loc_cen_y + radius);
+        if (y_hi > ny) y_hi = ny;
+        for (x = x_lo; x < x_hi; x++) {
+            for (y = y_lo; y < y_hi; y++) {
+                r = sqrt(pow(x - loc_cen_x,2) + pow(y - loc_cen_y,2));
+                if (r <= radius) pmask[x + y * nx] = 1.0;
+            }
+        }
+    } else {
+        /* Normalise mask to 1 and clip values below 0.5 */
+        cpl_image_divide_scalar(*out_mask, cpl_image_get_max(*out_mask));
+        for (i = 1; i <= cpl_image_get_size_x(*out_mask); i++) {
+            for (j = 1; j <= cpl_image_get_size_y(*out_mask); j++) {
+                if (cpl_image_get(*out_mask, i, j, &k) < 0.5) 
+                    cpl_image_set(*out_mask, i, j, 0.);
+                else   
+                    cpl_image_set(*out_mask, i, j, 1.);
+            }
         }
     }
 
@@ -1410,7 +1507,8 @@ static int kmos_std_star_compute_ifu(
                         sky_sky_struct.skyPairs[i].skyFrame1, STD, 
                         sky_sky_struct.skyPairs[i].skyFrame2, STD, flat_frame,
                         xcal_frame, ycal_frame, lcal_frame, NULL, NULL, &gd, 
-                        &tmp_cube, NULL, FALSE, FALSE, xcal_interpolation);
+                        &tmp_cube, NULL, FALSE, FALSE,
+                        xcal_interpolation, FALSE);
 
                 /* Extract spectrum using masked   */
                 /* convert mean to sum (* mask aperture) */
