@@ -210,10 +210,10 @@ static int kmos_wave_cal_create(cpl_plugin *plugin)
     recipe->parameters = cpl_parameterlist_new();
 
     // Fill the parameters list
-    p = cpl_parameter_new_value("kmos.kmos_wave_cal.order", CPL_TYPE_INT,
+    p = cpl_parameter_new_range("kmos.kmos_wave_cal.order", CPL_TYPE_INT,
             "The fitting polynomial order used for the wavelength solution. "
             "By default, 4 for IZ band, 5 for HK, 6 for the others",
-            "kmos.kmos_wave_cal", 0);
+            "kmos.kmos_wave_cal", 0, 0, 7);
     cpl_parameter_set_alias(p, CPL_PARAMETER_MODE_CLI, "order");
     cpl_parameter_disable(p, CPL_PARAMETER_MODE_ENV);
     cpl_parameterlist_append(recipe->parameters, p);
@@ -463,10 +463,14 @@ static int kmos_wave_cal(cpl_parameterlist *parlist, cpl_frameset *frameset)
     cpl_msg_info(__func__, "Arc lines: %lld", cpl_bivector_get_size(lines));
 
     /* Load REFLINES */
-    if (lines_estimation == 0) {
-        reflines = kmo_dfs_load_table(frameset, REF_LINES, 1, 0);
-    } else {
-        reflines = NULL ;
+    if ((reflines = kmo_dfs_load_table(frameset, REF_LINES, 1, 0)) == NULL) {
+        cpl_msg_error(__func__, "Missing REF_LINES calibration file") ;
+        cpl_error_set(__func__, CPL_ERROR_ILLEGAL_INPUT) ;
+        for (i = 0; i < next ; i++) cpl_free(filter_ids[i]);
+        cpl_free(filter_ids);
+        cpl_free(angles_array);
+        cpl_bivector_delete(lines) ;
+        return -1 ;
     }
 
     /* Check which IFUs are active for all FLAT frames */
@@ -717,7 +721,7 @@ static int kmos_wave_cal(cpl_parameterlist *parlist, cpl_frameset *frameset)
     cpl_free(stored_qc_arc_sat);
     cpl_free(stored_qc_ar_eff);
     cpl_free(stored_qc_ne_eff);
-    if (lines_estimation == 0) cpl_table_delete(reflines); 
+    cpl_table_delete(reflines); 
     
     /* QC parameters & saving */
     cpl_msg_info(__func__, "Saving data...");
