@@ -63,16 +63,16 @@ class DataPlotterManager(object):
             reflex.RecipeParameter(recipe=self.recipe_name, displayName="flux",
                     group="Extr.", description="Apply flux conservation"),
 
-            reflex.RecipeParameter(recipe=self.recipe_name, displayName="smethod",
-                    group="Other", description="Method to use for interpolation during shifting (NN, CS)"),
+            reflex.RecipeParameter(recipe=self.recipe_name, displayName="obj_sky_table",
+                    group="Sky Sub.", description="The path to the file with the modified obj/sky associations."),
             reflex.RecipeParameter(recipe=self.recipe_name, displayName="background",
-                    group="Other", description="Apply background removal"),
+                    group="Sky Sub.", description="Apply background removal"),
             reflex.RecipeParameter(recipe=self.recipe_name, displayName="no_subtract",
-                    group="Other", description="Don't sky subtract object and references"),
-            reflex.RecipeParameter(recipe=self.recipe_name, displayName="method",
-                    group="Other", description="The shifting method"),
-            reflex.RecipeParameter(recipe=self.recipe_name, displayName="filename",
-                    group="Other", description="The path to the file with the shift vectors"),
+                    group="Sky Sub.", description="Don't sky subtract object and references"),
+            reflex.RecipeParameter(recipe=self.recipe_name, displayName="sky_tweak",
+                    group="Sky Sub.", description="Use modified sky cube for sky subtraction.(TRUE (apply) or FALSE (don't apply)."),
+            reflex.RecipeParameter(recipe=self.recipe_name, displayName="velocity_offset",
+                    group="Sky Sub.", description="Specify velocity offset correction in km/s for lambda scale."),
         ]
 
     def readFitsData(self, fitsFiles):
@@ -130,6 +130,7 @@ class DataPlotterManager(object):
                         self.files[filename][extname]["CRVAL3"] = recons_ext.header['CRVAL3']
                         self.files[filename][extname]["CDELT3"] = recons_ext.header['CDELT3']
                         self.files[filename][extname]["UNIT"] = recons_ext.header['ESO QC CUBE_UNIT']
+                        self.files[filename][extname]["NAME"] = recons_ext.header['ESO OCS ARM' + ifu_number +' NAME']
                         
                         # Fill Spectrum
                         self.files[filename][extname]["Spectrum"] = []
@@ -185,11 +186,8 @@ class DataPlotterManager(object):
         widgets = list()
 
         # Files Selector radiobutton
-
-	#print(self.files.keys())
-
         self.radiobutton = reflex_plot_widgets.InteractiveRadioButtons(self.files_selector, self.setFSCallback, self.files.keys(), 0, 
-                title='Files Selection (Left mouse button)')
+                title='Files Selection (Left Mouse Button)')
         widgets.append(self.radiobutton)
         
         self.clickable_ifus =reflex_plot_widgets.InteractiveClickableSubplot(self.ifus_selector, self.setIFUSCallback)
@@ -219,8 +217,6 @@ class DataPlotterManager(object):
     def setFSCallback(self, filename) :
         # Keep track of the selected file
         self.selected_file = filename
-	
-	#debug print(self.selected_file)
 
         # Check that the new file currently selected extension is valid
         if (not self.extension_has_spectrum(self.selected_file, self.selected_extension)):
@@ -239,7 +235,7 @@ class DataPlotterManager(object):
         self.collapsed_img = figure.add_subplot(gs[2,1])
 
     def _data_plot_get_tooltip(self):
-        return self.selected_file+" ["+self.selected_extension+"]"
+        return self.selected_file+" ["+self.selected_extension+"];" +" Object NAME = "+str(  self.files[self.selected_file][self.selected_extension]["NAME"]  )
 
     def _disp_image(self):
         extension_dict = self.files[self.selected_file][self.selected_extension] 
@@ -250,26 +246,43 @@ class DataPlotterManager(object):
         # Only if data
         if 'Collapsed' in extension_dict.keys():
             imgdisp.display(self.collapsed_img, "Collapsed image", self._data_plot_get_tooltip(), extension_dict["Collapsed"])
+            self.collapsed_img.set_xlabel("pixels")   # item 6f
+            self.collapsed_img.set_ylabel("pixels")   # item 6f
+            
 
     def _plot_spectrum(self):
         extension_dict = self.files[self.selected_file][self.selected_extension] 
         
+        # Plot Spectrum (to be contd.)
+        self.spec_plot.clear()
+        specdisp = pipeline_display.SpectrumDisplay()
+        specdisp.setLabels(r"$\lambda$["+"$\mu$m]" + " (blue: Observed; red: OH[arb. units])", 
+                            self._process_label(extension_dict["UNIT"]) )
+        
+        if (self.oh_spec["Found"]):  # Placed before spectrum plotting, so it stays in the background
+			# Overplot the OH spectrum
+            pix = numpy.arange(len(self.oh_spec["Spectrum"]))
+            wave = self.oh_spec["CRVAL1"] + pix * self.oh_spec["CDELT1"]
+            oh_flux = self.oh_spec["Spectrum"] / numpy.nanmax(self.oh_spec["Spectrum"]) * 200*numpy.nanmax(extension_dict["Spectrum"]) 
+            specdisp.overplot(self.spec_plot, wave, oh_flux, '#ED5D5D')
+            #self.spec_plot.legend(('Observed', 'OH'))
+            self.spec_plot.fill_between(wave, 0, oh_flux, color='#ED5D5D')
+        
         # Define wave
         pix = numpy.arange(len(extension_dict["Spectrum"]))
         wave = extension_dict["CRVAL3"] + pix * extension_dict["CDELT3"]
 
-        # Plot Spectrum
-        self.spec_plot.clear()
-        specdisp = pipeline_display.SpectrumDisplay()
-        specdisp.setLabels("Wavelength (microns)", extension_dict["UNIT"])
+		# Plot Spectrum (contd.)
         specdisp.display(self.spec_plot, "Spectrum", self._data_plot_get_tooltip(), wave, extension_dict["Spectrum"])
 
-        if (self.oh_spec["Found"]):
-            # Overplot the OH spectrum
-            pix = numpy.arange(len(self.oh_spec["Spectrum"]))
-            wave = self.oh_spec["CRVAL1"] + pix * self.oh_spec["CDELT1"]
-            specdisp.overplot(self.spec_plot, wave, self.oh_spec["Spectrum"], 'red')
-            self.spec_plot.legend(('Observed', 'OH'))
+
+
+    def _process_label(self, in_label):
+        # If known, 'pretty print' the label
+        if (in_label == "ERG/sec/cm2/A"):
+            return "erg sec" + r"$^{-1}$"+"cm" + r"$^{-2}$" + r"$\AA^{-1}$"
+        else:
+            return in_label
 
     def _get_ifu_status_from_file(self, filename, extname):
         cur_status = self.files[filename][extname]["IFU_STATUS"]
